@@ -29,7 +29,8 @@ class SparkException(
     cause: Throwable,
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext] = Array.empty)
+    context: Array[QueryContext] = Array.empty,
+    sqlState: Option[String] = None)
   extends Exception(message, cause) with SparkThrowable {
 
   def this(message: String, cause: Throwable) =
@@ -38,6 +39,15 @@ class SparkException(
   def this(message: String) =
     this(message = message, cause = null)
 
+  def this(
+      message: String,
+      cause: Throwable,
+      errorClass: Option[String],
+      messageParameters: Map[String, String],
+      context: Array[QueryContext]
+  ) =
+    this(message, cause, errorClass, messageParameters, context, None)
+
   def this(
       errorClass: String,
       messageParameters: Map[String, String],
@@ -71,6 +81,8 @@ class SparkException(
 
   override def getCondition: String = errorClass.orNull
 
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -170,7 +182,8 @@ private[spark] class SparkUpgradeException private(
     message: String,
     cause: Option[Throwable],
     errorClass: Option[String],
-    messageParameters: Map[String, String])
+    messageParameters: Map[String, String],
+    sqlState: Option[String] = None)
   extends RuntimeException(message, cause.orNull) with SparkThrowable {
 
   def this(
@@ -185,9 +198,25 @@ private[spark] class SparkUpgradeException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    cause: Throwable,
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters),
+      Option(cause),
+      Option(errorClass),
+      messageParameters,
+      sqlState
+    )
+  }
+
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -197,7 +226,8 @@ private[spark] class SparkArithmeticException private(
     message: String,
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext])
+    context: Array[QueryContext],
+    sqlState: Option[String] = None)
   extends ArithmeticException(message) with SparkThrowable {
 
   def this(
@@ -213,6 +243,20 @@ private[spark] class SparkArithmeticException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    context: Array[QueryContext],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, ""),
+      Option(errorClass),
+      messageParameters,
+      context,
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: Map[String, String],
@@ -221,6 +265,9 @@ private[spark] class SparkArithmeticException private(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -230,7 +277,8 @@ private[spark] class SparkArithmeticException private(
 private[spark] class SparkUnsupportedOperationException private(
   message: String,
   errorClass: Option[String],
-  messageParameters: Map[String, String])
+  messageParameters: Map[String, String],
+  sqlState: Option[String] = None)
   extends UnsupportedOperationException(message) with SparkThrowable {
 
   def this(
@@ -243,6 +291,18 @@ private[spark] class SparkUnsupportedOperationException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters),
+      Option(errorClass),
+      messageParameters,
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: java.util.Map[String, String]) =
@@ -253,12 +313,15 @@ private[spark] class SparkUnsupportedOperationException private(
     this(
       SparkThrowableHelper.getMessage(errorClass, Map.empty[String, String]),
       Option(errorClass),
-      Map.empty)
+      Map.empty,
+      None)
   }
 
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 private[spark] object SparkUnsupportedOperationException {
@@ -281,7 +344,8 @@ private[spark] object SparkUnsupportedOperationException {
 private[spark] class SparkClassNotFoundException(
     errorClass: String,
     messageParameters: Map[String, String],
-    cause: Throwable = null)
+    cause: Throwable = null,
+    sqlState: Option[String] = None)
   extends ClassNotFoundException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters), cause)
   with SparkThrowable {
@@ -289,6 +353,8 @@ private[spark] class SparkClassNotFoundException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -297,7 +363,8 @@ private[spark] class SparkClassNotFoundException(
 private[spark] class SparkConcurrentModificationException(
     errorClass: String,
     messageParameters: Map[String, String],
-    cause: Throwable = null)
+    cause: Throwable = null,
+    sqlState: Option[String] = None)
   extends ConcurrentModificationException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters), cause)
   with SparkThrowable {
@@ -305,6 +372,8 @@ private[spark] class SparkConcurrentModificationException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -315,7 +384,8 @@ private[spark] class SparkDateTimeException private(
     errorClass: Option[String],
     messageParameters: Map[String, String],
     context: Array[QueryContext],
-    cause: Option[Throwable])
+    cause: Option[Throwable],
+    sqlState: Option[String] = None)
   extends DateTimeException(message, cause.orNull) with SparkThrowable {
 
   def this(
@@ -347,6 +417,23 @@ private[spark] class SparkDateTimeException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    context: Array[QueryContext],
+    summary: String,
+    cause: Option[Throwable],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, summary),
+      Option(errorClass),
+      messageParameters,
+      context,
+      cause.orElse(None),
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: Map[String, String],
@@ -355,6 +442,9 @@ private[spark] class SparkDateTimeException private(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -363,7 +453,8 @@ private[spark] class SparkDateTimeException private(
  */
 private[spark] class SparkFileNotFoundException(
     errorClass: String,
-    messageParameters: Map[String, String])
+    messageParameters: Map[String, String],
+    sqlState: Option[String] = None)
   extends FileNotFoundException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters))
   with SparkThrowable {
@@ -371,6 +462,8 @@ private[spark] class SparkFileNotFoundException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -380,7 +473,8 @@ private[spark] class SparkNumberFormatException private(
     message: String,
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext])
+    context: Array[QueryContext],
+    sqlState: Option[String] = None)
   extends NumberFormatException(message)
   with SparkThrowable {
 
@@ -397,6 +491,20 @@ private[spark] class SparkNumberFormatException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    context: Array[QueryContext],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, ""),
+      Option(errorClass),
+      messageParameters,
+      context,
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: Map[String, String],
@@ -405,6 +513,9 @@ private[spark] class SparkNumberFormatException private(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -416,7 +527,8 @@ private[spark] class SparkIllegalArgumentException private(
     cause: Option[Throwable],
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext])
+    context: Array[QueryContext],
+    sqlState: Option[String] = None)
   extends IllegalArgumentException(message, cause.orNull)
   with SparkThrowable {
 
@@ -435,6 +547,23 @@ private[spark] class SparkIllegalArgumentException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    context: Array[QueryContext],
+    summary: String,
+    cause: Throwable,
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, summary),
+      Option(cause),
+      Option(errorClass),
+      messageParameters,
+      context,
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: Map[String, String],
@@ -457,6 +586,9 @@ private[spark] class SparkIllegalArgumentException private(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -467,7 +599,8 @@ private[spark] class SparkIllegalStateException(
     errorClass: String,
     messageParameters: Map[String, String],
     context: Array[QueryContext] = Array.empty,
-    cause: Throwable = null)
+    cause: Throwable = null,
+    sqlState: Option[String] = None)
   extends IllegalStateException(
       SparkThrowableHelper.getMessage(errorClass, messageParameters), cause)
     with SparkThrowable {
@@ -476,6 +609,8 @@ private[spark] class SparkIllegalStateException(
 
   override def getCondition: String = errorClass
 
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -484,7 +619,8 @@ private[spark] class SparkRuntimeException private(
     cause: Option[Throwable],
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext])
+    context: Array[QueryContext],
+    sqlState: Option[String])
   extends RuntimeException(message, cause.orNull) with SparkThrowable {
 
   def this(
@@ -498,13 +634,33 @@ private[spark] class SparkRuntimeException private(
       Option(cause),
       Option(errorClass),
       messageParameters,
-      context
+      context,
+      None
+    )
+  }
+
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    cause: Throwable,
+    context: Array[QueryContext],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, ""),
+      Option(cause),
+      Option(errorClass),
+      messageParameters,
+      context,
+      sqlState
     )
   }
 
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -513,7 +669,8 @@ private[spark] class SparkPythonException private(
     cause: Option[Throwable],
     errorClass: Option[String],
     messageParameters: Map[String, String],
-    context: Array[QueryContext])
+    context: Array[QueryContext],
+    sqlState: Option[String])
   extends RuntimeException(message, cause.orNull) with SparkThrowable {
 
   def this(
@@ -527,13 +684,17 @@ private[spark] class SparkPythonException private(
       Option(cause),
       Option(errorClass),
       messageParameters,
-      context
+      context,
+      None
     )
   }
 
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -544,7 +705,8 @@ private[spark] class SparkNoSuchElementException(
     errorClass: String,
     messageParameters: Map[String, String],
     context: Array[QueryContext] = Array.empty,
-    summary: String = "")
+    summary: String = "",
+    sqlState: Option[String] = None)
     extends NoSuchElementException(
       SparkThrowableHelper.getMessage(errorClass, messageParameters, summary))
     with SparkThrowable {
@@ -553,6 +715,8 @@ private[spark] class SparkNoSuchElementException(
 
   override def getCondition: String = errorClass
 
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -561,7 +725,8 @@ private[spark] class SparkNoSuchElementException(
  */
 private[spark] class SparkSecurityException(
     errorClass: String,
-    messageParameters: Map[String, String])
+    messageParameters: Map[String, String],
+    sqlState: Option[String] = None)
   extends SecurityException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters))
   with SparkThrowable {
@@ -569,6 +734,8 @@ private[spark] class SparkSecurityException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -578,7 +745,8 @@ private[spark] class SparkArrayIndexOutOfBoundsException private(
   message: String,
   errorClass: Option[String],
   messageParameters: Map[String, String],
-  context: Array[QueryContext])
+  context: Array[QueryContext],
+  sqlState: Option[String] = None)
   extends ArrayIndexOutOfBoundsException(message)
     with SparkThrowable {
 
@@ -595,6 +763,20 @@ private[spark] class SparkArrayIndexOutOfBoundsException private(
     )
   }
 
+  def this(
+    errorClass: String,
+    messageParameters: Map[String, String],
+    context: Array[QueryContext],
+    sqlState: Option[String]) = {
+    this(
+      SparkThrowableHelper.getMessage(errorClass, messageParameters, ""),
+      Option(errorClass),
+      messageParameters,
+      context,
+      sqlState
+    )
+  }
+
   def this(
     errorClass: String,
     messageParameters: Map[String, String],
@@ -603,6 +785,9 @@ private[spark] class SparkArrayIndexOutOfBoundsException private(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass.orNull
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
+
   override def getQueryContext: Array[QueryContext] = context
 }
 
@@ -611,7 +796,8 @@ private[spark] class SparkArrayIndexOutOfBoundsException private(
  */
 private[spark] class SparkSQLException(
     errorClass: String,
-    messageParameters: Map[String, String])
+    messageParameters: Map[String, String],
+    sqlState: Option[String] = None)
   extends SQLException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters))
   with SparkThrowable {
@@ -619,6 +805,8 @@ private[spark] class SparkSQLException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
 
 /**
@@ -626,7 +814,8 @@ private[spark] class SparkSQLException(
  */
 private[spark] class SparkSQLFeatureNotSupportedException(
     errorClass: String,
-    messageParameters: Map[String, String])
+    messageParameters: Map[String, String],
+    sqlState: Option[String] = None)
   extends SQLFeatureNotSupportedException(
     SparkThrowableHelper.getMessage(errorClass, messageParameters))
   with SparkThrowable {
@@ -634,4 +823,6 @@ private[spark] class SparkSQLFeatureNotSupportedException(
   override def getMessageParameters: java.util.Map[String, String] = messageParameters.asJava
 
   override def getCondition: String = errorClass
+
+  override def getSqlState: String = sqlState.getOrElse(super.getSqlState)
 }
