@@ -29,11 +29,12 @@ import org.apache.avro.io.{DecoderFactory, EncoderFactory}
 import org.apache.spark.SparkException
 import org.apache.spark.sql.{AnalysisException, QueryTest, Row}
 import org.apache.spark.sql.avro.{functions => Fns}
+import org.apache.spark.sql.avro.functions.{from_avro, to_avro}
 import org.apache.spark.sql.execution.LocalTableScanExec
 import org.apache.spark.sql.functions.{col, lit, struct}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.test.SharedSparkSession
-import org.apache.spark.sql.types.{BinaryType, StructType}
+import org.apache.spark.sql.types.{BinaryType, IntegerType, StructField, StructType}
 
 class AvroFunctionsSuite extends QueryTest with SharedSparkSession {
   import testImplicits._
@@ -106,6 +107,17 @@ class AvroFunctionsSuite extends QueryTest with SharedSparkSession {
        functions.from_avro(
           $"avro", avroTypeStruct, Map("mode" -> "PERMISSIVE").asJava)),
       expected)
+
+    checkError(
+      exception = intercept[AnalysisException] {
+        avroStructDF.select(
+          functions.from_avro(
+            $"avro", avroTypeStruct, Map("mode" -> "DROPMALFORMED").asJava)).collect()
+      },
+      condition = "PARSE_MODE_UNSUPPORTED",
+      parameters = Map(
+        "funcName" -> "`from_avro`",
+        "mode" -> "DROPMALFORMED"))
   }
 
   test("roundtrip in to_avro and from_avro - array with null") {
@@ -374,6 +386,37 @@ class AvroFunctionsSuite extends QueryTest with SharedSparkSession {
     }
   }
 
+
+  test("roundtrip in to_avro and from_avro - recursive schema") {
+    val catalystSchema =
+      StructType(Seq(
+        StructField("Id", IntegerType),
+        StructField("Name", StructType(Seq(
+          StructField("Id", IntegerType),
+          StructField("Name", StructType(Seq(
+            StructField("Id", IntegerType)))))))))
+
+    val avroSchema = s"""
+                        |{
+                        |  "type" : "record",
+                        |  "name" : "test_schema",
+                        |  "fields" : [
+                        |    {"name": "Id", "type": "int"},
+                        |    {"name": "Name", "type": ["null", "test_schema"]}
+                        |  ]
+                        |}
+    """.stripMargin
+
+    val df = spark.createDataFrame(
+      spark.sparkContext.parallelize(Seq(Row(2, Row(3, Row(4))), Row(1, null))),
+      catalystSchema).select(struct("Id", "Name").as("struct"))
+
+    val avroStructDF = df.select(functions.to_avro($"struct", avroSchema).as("avro"))
+    checkAnswer(avroStructDF.select(
+      functions.from_avro($"avro", avroSchema, Map(
+        "recursiveFieldMaxDepth" -> "3").asJava)), df)
+  }
+
   private def serialize(record: GenericRecord, avroSchema: String): Array[Byte] = {
     val schema = new Schema.Parser().parse(avroSchema)
     val datumWriter = new GenericDatumWriter[GenericRecord](schema)
@@ -587,4 +630,117 @@ class AvroFunctionsSuite extends QueryTest with SharedSparkSession {
       assert(readbackPerson2.get(2).toString === person2.get(2))
     }
   }
+
+  test("schema_of_avro") {
+    val df = spark.range(1)
+    val avroIntType = s"""
+      |{
+      |  "type": "int",
+      |  "name": "id"
+      |}""".stripMargin
+    checkAnswer(df.select(functions.schema_of_avro(avroIntType)), Row("INT"))
+
+    val avroStructType =
+      """
+        |{
+        |  "type": "record",
+        |  "name": "person",
+        |  "fields": [
+        |    {"name": "name", "type": "string"},
+        |    {"name": "age", "type": "int"},
+        |    {"name": "country", "type": "string"}
+        |  ]
+        |}""".stripMargin
+    checkAnswer(df.select(functions.schema_of_avro(avroStructType)),
+      Row("STRUCT<name: STRING NOT NULL, age: INT NOT NULL, country: STRING NOT NULL>"))
+
+    val avroMultiType =
+      """
+        |{
+        |  "type": "record",
+        |  "name": "person",
+        |  "fields": [
+        |     {"name": "u", "type": ["int", "string"]}
+        |  ]
+        |}""".stripMargin
+    checkAnswer(df.select(functions.schema_of_avro(avroMultiType)),
+      Row("STRUCT<u: STRUCT<member0: INT, member1: STRING> NOT NULL>"))
+  }
+
+  test("roundtrip in to_avro and from_avro - TIME type with different precisions") {
+    val df = spark.sql("""
+      SELECT
+        TIME'12:34:56' as time_p0,
+        TIME'12:34:56.1' as time_p1,
+        TIME'12:34:56.12' as time_p2,
+        TIME'12:34:56.123' as time_p3,
+        TIME'12:34:56.1234' as time_p4,
+        TIME'12:34:56.12345' as time_p5,
+        TIME'12:34:56.123456' as time_p6
+    """)
+
+    val precisions = Seq(0, 1, 2, 3, 4, 5, 6)
+    precisions.foreach { p =>
+      val fieldName = s"time_p$p"
+      // Generate correct schema for each precision
+      val avroTimeSchema = s"""
+        |{
+        |  "type": "long",
+        |  "logicalType": "time-micros",
+        |  "spark.sql.catalyst.type": "time($p)"
+        |}
+      """.stripMargin
+
+      val avroDF = df.select(to_avro(col(fieldName)).as("avro"))
+      val readBack = avroDF.select(from_avro($"avro", avroTimeSchema).as(fieldName))
+
+      checkAnswer(readBack, df.select(col(fieldName)))
+    }
+  }
+
+  test("roundtrip in to_avro and from_avro - TIME type in struct") {
+    val df = spark.sql("""
+      SELECT
+        struct(
+          TIME'09:00:00.123' as start,
+          TIME'17:30:45.987654' as end,
+          'Morning Shift' as description
+        ) as schedule
+    """)
+
+    val avroStructDF = df.select(to_avro($"schedule").as("avro"))
+
+    val avroStructSchema = """
+      |{
+      |  "type": "record",
+      |  "name": "schedule",
+      |  "fields": [
+      |    {
+      |      "name": "start",
+      |      "type": {
+      |        "type": "long",
+      |        "logicalType": "time-micros",
+      |        "spark.sql.catalyst.type": "time(3)"
+      |      }
+      |    },
+      |    {
+      |      "name": "end",
+      |      "type": {
+      |        "type": "long",
+      |        "logicalType": "time-micros",
+      |        "spark.sql.catalyst.type": "time(6)"
+      |      }
+      |    },
+      |    {
+      |      "name": "description",
+      |      "type": "string"
+      |    }
+      |  ]
+      |}
+    """.stripMargin
+
+    val readBack = avroStructDF.select(
+      from_avro($"avro", avroStructSchema).as("schedule"))
+    checkAnswer(readBack, df)
+  }
 }
