@@ -582,12 +582,6 @@ class FrameConstructorTests(
 
 
 if __name__ == "__main__":
-    from pyspark.pandas.tests.frame.test_constructor import *  # noqa: F401
+    from pyspark.testing import main
 
-    try:
-        import xmlrunner
-
-        testRunner = xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2)
-    except ImportError:
-        testRunner = None
-    unittest.main(testRunner=testRunner, verbosity=2)
+    main()
