@@ -19,23 +19,18 @@ package org.apache.spark.sql.streaming
 
 import java.io.Serializable
 
-import org.apache.spark.annotation.{Evolving, Experimental}
+import org.apache.spark.annotation.Evolving
 
 /**
- * Class used to provide access to expired timer's expiry time. These values are only relevant if
- * the ExpiredTimerInfo is valid.
+ * Class used to provide access to expired timer's expiry time.
  */
-@Experimental
 @Evolving
-private[sql] trait ExpiredTimerInfo extends Serializable {
+trait ExpiredTimerInfo extends Serializable {
 
   /**
-   * Check if provided ExpiredTimerInfo is valid.
-   */
-  def isValid(): Boolean
-
-  /**
-   * Get the expired timer's expiry time as milliseconds in epoch time.
+   * Function to return the expired timer's expiry time as milliseconds in epoch time.
+   *
+   * @return - the expired timer's expiry time in milliseconds
    */
   def getExpiryTimeInMs(): Long
 }
