@@ -18,12 +18,15 @@
 package org.apache.spark.sql.catalyst.csv
 
 import java.io.InputStream
+import java.util.Locale
 
+import scala.collection.mutable.ArrayBuffer
 import scala.util.control.NonFatal
 
+import com.univocity.parsers.common.TextParsingException
 import com.univocity.parsers.csv.CsvParser
 
-import org.apache.spark.SparkUpgradeException
+import org.apache.spark.{SparkRuntimeException, SparkUpgradeException}
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.catalyst.{InternalRow, NoopFilters, OrderedFilters}
 import org.apache.spark.sql.catalyst.expressions.{ExprUtils, GenericInternalRow}
@@ -33,7 +36,8 @@ import org.apache.spark.sql.errors.{ExecutionErrors, QueryExecutionErrors}
 import org.apache.spark.sql.internal.{LegacyBehaviorPolicy, SQLConf}
 import org.apache.spark.sql.sources.Filter
 import org.apache.spark.sql.types._
-import org.apache.spark.unsafe.types.UTF8String
+import org.apache.spark.types.variant._
+import org.apache.spark.unsafe.types.{UTF8String, VariantVal}
 
 /**
  * Constructs a parser for a given schema that translates CSV data to an [[InternalRow]].
@@ -109,6 +113,7 @@ class UnivocityParser(
     options.locale,
     legacyFormat = FAST_DATE_FORMAT,
     isParsing = true)
+  private lazy val timeFormatter = TimeFormatter(options.timeFormatInRead, isParsing = true)
 
   private val csvFilters = if (SQLConf.get.csvFilterPushDown) {
     new OrderedFilters(filters, requiredSchema)
@@ -134,6 +139,11 @@ class UnivocityParser(
           options.dateFormatOption.isEmpty
       }
 
+  // When `options.needHeaderForSingleVariantColumn` is true, it will be set to the header column
+  // names by `CSVDataSource.readHeaderForSingleVariantColumn`.
+  var headerColumnNames: Option[Array[String]] = None
+  private val singleVariantFieldConverters = new ArrayBuffer[VariantValueConverter]()
+
   // Retrieve the raw record string.
   private def getCurrentInput: UTF8String = {
     if (tokenizer.getContext == null) return null
@@ -160,9 +170,12 @@ class UnivocityParser(
   // Each input token is placed in each output row's position by mapping these. In this case,
   //
   //   output row - ["A", 2]
-  private val valueConverters: Array[ValueConverter] = {
-    requiredSchema.map(f => makeConverter(f.name, f.dataType, f.nullable)).toArray
-  }
+  private val valueConverters: Array[ValueConverter] =
+    if (options.singleVariantColumn.isDefined) {
+      null
+    } else {
+      requiredSchema.map(f => makeConverter(f.name, f.dataType, f.nullable)).toArray
+    }
 
   private val decimalParser = ExprUtils.getDecimalParser(options.locale)
 
@@ -250,6 +263,11 @@ class UnivocityParser(
         timestampNTZFormatter.parseWithoutTimeZone(datum, false)
       }
 
+    case _: TimeType => (d: String) =>
+      nullSafeDatum(d, name, nullable, options) { datum =>
+        timeFormatter.parse(datum)
+      }
+
     case _: StringType => (d: String) =>
       nullSafeDatum(d, name, nullable, options)(UTF8String.fromString)
 
@@ -273,6 +291,8 @@ class UnivocityParser(
           UTF8String.fromString(datum), dt.startField, dt.endField)
       }
 
+    case _: VariantType => new VariantValueConverter
+
     case udt: UserDefinedType[_] =>
       makeConverter(name, udt.sqlType, nullable)
 
@@ -294,6 +314,20 @@ class UnivocityParser(
     }
   }
 
+  private def parseLine(line: String): Array[String] = {
+    try {
+      tokenizer.parseLine(line)
+    }
+    catch {
+      case e: TextParsingException if e.getCause.isInstanceOf[ArrayIndexOutOfBoundsException] =>
+        throw new SparkRuntimeException(
+          errorClass = "MALFORMED_CSV_RECORD",
+          messageParameters = Map("badRecord" -> line),
+          cause = e
+        )
+    }
+  }
+
   /**
    * Parses a single CSV string and turns it into either one resulting row or no row (if the
    * the record is malformed).
@@ -306,7 +340,7 @@ class UnivocityParser(
       (_: String) => Some(InternalRow.empty)
     } else {
       // parse if the columnPruning is disabled or requiredSchema is nonEmpty
-      (input: String) => convert(tokenizer.parseLine(input))
+      (input: String) => convert(parseLine(input))
     }
   }
 
@@ -316,6 +350,46 @@ class UnivocityParser(
     (tokens: Array[String], index: Int) => tokens(tokenIndexArr(index))
   }
 
+  /**
+   * The entire line of CSV data is collected into a single variant object. When `headerColumnNames`
+   * is defined, the field names will be extracted from it. Otherwise, the field names will have a
+   * a format of "_c$i" to match the position of the values in the CSV data.
+   */
+  protected final def convertSingleVariantRow(
+      tokens: Array[String],
+      currentInput: UTF8String): GenericInternalRow = {
+    val row = new GenericInternalRow(1)
+    try {
+      val keys = headerColumnNames.orNull
+      val numFields = if (keys != null) tokens.length.min(keys.length) else tokens.length
+      if (singleVariantFieldConverters.length < numFields) {
+        val extra = numFields - singleVariantFieldConverters.length
+        singleVariantFieldConverters.appendAll(Array.fill(extra)(new VariantValueConverter))
+      }
+      val builder = new VariantBuilder(false)
+      val start = builder.getWritePos
+      val fields = new java.util.ArrayList[VariantBuilder.FieldEntry](numFields)
+      for (i <- 0 until numFields) {
+        val key = if (keys != null) keys(i) else "_c" + i
+        val id = builder.addKey(key)
+        fields.add(new VariantBuilder.FieldEntry(key, id, builder.getWritePos - start))
+        singleVariantFieldConverters(i).convertInput(builder, tokens(i))
+      }
+      builder.finishWritingObject(start, fields)
+      val v = builder.result()
+      row(0) = new VariantVal(v.getValue, v.getMetadata)
+      // If the header line has different number of tokens than the content line, the CSV data is
+      // malformed. We may still have partially parsed data in `row`.
+      if (keys != null && keys.length != tokens.length) {
+        throw QueryExecutionErrors.malformedCSVRecordError(currentInput.toString)
+      }
+      row
+    } catch {
+      case NonFatal(e) =>
+        throw BadRecordException(() => currentInput, () => Array(row), cause = e)
+    }
+  }
+
   private def convert(tokens: Array[String]): Option[InternalRow] = {
     if (tokens == null) {
       throw BadRecordException(
@@ -326,6 +400,10 @@ class UnivocityParser(
 
     val currentInput = getCurrentInput
 
+    if (options.singleVariantColumn.isDefined) {
+      return Some(convertSingleVariantRow(tokens, currentInput))
+    }
+
     var badRecordException: Option[Throwable] = if (tokens.length != parsedSchema.length) {
       // If the number of tokens doesn't match the schema, we should treat it as a malformed record.
       // However, we still have chance to parse some of the tokens. It continues to parses the
@@ -371,6 +449,132 @@ class UnivocityParser(
       }
     }
   }
+
+  /**
+   * This class converts a comma-separated value into a variant column (when the schema contains
+   * variant type) or a variant field (when in singleVariantColumn mode).
+   *
+   * It has a list of scalar types to try (long, decimal, date, timestamp, boolean) and maintains
+   * the current content type. It tries to parse the input as the current content type. If the
+   * parsing fails, it moves to the next type in the list and continues the trial. It never checks
+   * the previous types that have already failed. In the end, it either successfully parses the
+   * input as a specific scalar type, or fails after trying all the types and defaults to the string
+   * type. The state is reset for every input file.
+   *
+   * Floating point types (double, float) are not considered to avoid precision loss.
+   */
+  private final class VariantValueConverter extends ValueConverter {
+    private var currentType: DataType = LongType
+    // Keep consistent with `CSVInferSchema`: only produce TimestampNTZ when the default timestamp
+    // type is TimestampNTZ.
+    private val isDefaultNTZ = SQLConf.get.timestampType == TimestampNTZType
+
+    override def apply(s: String): Any = {
+      val builder = new VariantBuilder(false)
+      convertInput(builder, s)
+      val v = builder.result()
+      new VariantVal(v.getValue, v.getMetadata)
+    }
+
+    def convertInput(builder: VariantBuilder, s: String): Unit = {
+      if (s == null || s == options.nullValue) {
+        builder.appendNull()
+        return
+      }
+
+      def parseLong(): DataType = {
+        try {
+          builder.appendLong(s.toLong)
+          // The actual integral type doesn't matter. `appendLong` will use the smallest possible
+          // integral type to store the value.
+          LongType
+        } catch {
+          case NonFatal(_) => parseDecimal()
+        }
+      }
+
+      def parseDecimal(): DataType = {
+        try {
+          var d = decimalParser(s)
+          if (d.scale() < 0) {
+            d = d.setScale(0)
+          }
+          if (d.scale() <= VariantUtil.MAX_DECIMAL16_PRECISION &&
+            d.precision() <= VariantUtil.MAX_DECIMAL16_PRECISION) {
+            builder.appendDecimal(d)
+            // The actual decimal type doesn't matter. `appendDecimal` will use the smallest
+            // possible decimal type to store the value.
+            DecimalType.USER_DEFAULT
+          } else {
+            if (options.preferDate) parseDate() else parseTimestampNTZ()
+          }
+        } catch {
+          case NonFatal(_) =>
+            if (options.preferDate) parseDate() else parseTimestampNTZ()
+        }
+      }
+
+      def parseDate(): DataType = {
+        try {
+          builder.appendDate(dateFormatter.parse(s))
+          DateType
+        } catch {
+          case NonFatal(_) => parseTimestampNTZ()
+        }
+      }
+
+      def parseTimestampNTZ(): DataType = {
+        if (isDefaultNTZ) {
+          try {
+            builder.appendTimestampNtz(timestampNTZFormatter.parseWithoutTimeZone(s, false))
+            TimestampNTZType
+          } catch {
+            case NonFatal(_) => parseTimestamp()
+          }
+        } else {
+          parseTimestamp()
+        }
+      }
+
+      def parseTimestamp(): DataType = {
+        try {
+          builder.appendTimestamp(timestampFormatter.parse(s))
+          TimestampType
+        } catch {
+          case NonFatal(_) => parseBoolean()
+        }
+      }
+
+      def parseBoolean(): DataType = {
+        val lower = s.toLowerCase(Locale.ROOT)
+        if (lower == "true") {
+          builder.appendBoolean(true)
+          BooleanType
+        } else if (lower == "false") {
+          builder.appendBoolean(false)
+          BooleanType
+        } else {
+          parseString()
+        }
+      }
+
+      def parseString(): DataType = {
+        builder.appendString(s)
+        StringType
+      }
+
+      val newType = currentType match {
+        case LongType => parseLong()
+        case _: DecimalType => parseDecimal()
+        case DateType => parseDate()
+        case TimestampNTZType => parseTimestampNTZ()
+        case TimestampType => parseTimestamp()
+        case BooleanType => parseBoolean()
+        case StringType => parseString()
+      }
+      currentType = newType
+    }
+  }
 }
 
 private[sql] object UnivocityParser {
