@@ -24,11 +24,11 @@ import scala.util.matching.Regex
 
 import org.antlr.v4.runtime.{ParserRuleContext, Token}
 import org.antlr.v4.runtime.misc.Interval
-import org.antlr.v4.runtime.tree.{ParseTree, TerminalNodeImpl}
+import org.antlr.v4.runtime.tree.{ParseTree, TerminalNode, TerminalNodeImpl}
 
 import org.apache.spark.SparkException
 import org.apache.spark.sql.catalyst.analysis.UnresolvedIdentifier
-import org.apache.spark.sql.catalyst.parser.SqlBaseParser.{BeginLabelContext, EndLabelContext, MultipartIdentifierContext}
+import org.apache.spark.sql.catalyst.parser.SqlBaseParser.{BeginLabelContext, EndLabelContext, StrictIdentifierContext}
 import org.apache.spark.sql.catalyst.plans.logical.{CreateVariable, ErrorCondition}
 import org.apache.spark.sql.catalyst.trees.CurrentOrigin
 import org.apache.spark.sql.catalyst.util.SparkParserUtils
@@ -88,6 +88,18 @@ object ParserUtils extends SparkParserUtils {
     node.getText.slice(1, node.getText.length - 1)
   }
 
+  /**
+   * Obtain the string literal provided as a dollar quoted string.
+   * A dollar quoted string is defined as {{{$[tag]$<string literal>$[tag]$}}},
+   * where the string literal is parsed as a list of body sections.
+   * This helper method concatenates all body sections and restores the string literal back.
+   */
+  def dollarQuotedString(sections: util.List[TerminalNode]): String = {
+    val sb = new StringBuilder()
+    sections forEach (body => sb.append(body.getText))
+    sb.toString()
+  }
+
   /** Collect the entries if any. */
   def entry(key: String, value: Token): Seq[(String, String)] = {
     Option(value).toSeq.map(x => key -> string(x))
@@ -142,7 +154,7 @@ class CompoundBodyParsingContext {
 
   object State extends Enumeration {
     type State = Value
-    val INIT, VARIABLE, CONDITION, HANDLER, STATEMENT = Value
+    val INIT, VARIABLE, CONDITION, CURSOR, HANDLER, STATEMENT = Value
   }
 
   private var currentState: State.State = State.INIT
@@ -167,6 +179,11 @@ class CompoundBodyParsingContext {
     transitionTo(State.CONDITION, None, errorCondition = Some(errorCondition))
   }
 
+  /** Transition to CURSOR state. */
+  def cursor(): Unit = {
+    transitionTo(State.CURSOR)
+  }
+
   /** Transition to HANDLER state. */
   def handler(): Unit = {
     transitionTo(State.HANDLER)
@@ -182,8 +199,9 @@ class CompoundBodyParsingContext {
    * Possible states are:
    * 1a. VARIABLE (1)
    * 1b. CONDITION (1)
-   * 2. HANDLERS (2)
-   * 3. STATEMENTS (3)
+   * 2. CURSOR (2)
+   * 3. HANDLERS (3)
+   * 4. STATEMENTS (4)
    * Transition is allowed from state with number n to state with number m,
    * where m >= n.
    *
@@ -203,6 +221,8 @@ class CompoundBodyParsingContext {
 
       case (State.VARIABLE, State.CONDITION) => currentState = State.CONDITION
 
+      case (State.VARIABLE, State.CURSOR) => currentState = State.CURSOR
+
       case (State.VARIABLE, State.HANDLER) => currentState = State.HANDLER
 
       case (State.VARIABLE, State.STATEMENT) => currentState = State.STATEMENT
@@ -212,10 +232,19 @@ class CompoundBodyParsingContext {
 
       case (State.CONDITION, State.VARIABLE) => currentState = State.VARIABLE
 
+      case (State.CONDITION, State.CURSOR) => currentState = State.CURSOR
+
       case (State.CONDITION, State.HANDLER) => currentState = State.HANDLER
 
       case (State.CONDITION, State.STATEMENT) => currentState = State.STATEMENT
 
+      // Transition from CURSOR to other states.
+      case (State.CURSOR, State.CURSOR) => // do nothing
+
+      case (State.CURSOR, State.HANDLER) => currentState = State.HANDLER
+
+      case (State.CURSOR, State.STATEMENT) => currentState = State.STATEMENT
+
       // Transition from HANDLER to other states.
       case (State.HANDLER, State.HANDLER) => // do nothing
 
@@ -227,6 +256,11 @@ class CompoundBodyParsingContext {
       // INVALID TRANSITIONS
 
       // Invalid transitions to VARIABLE state.
+      case (State.CURSOR, State.VARIABLE) =>
+        throw SqlScriptingErrors.variableDeclarationOnlyAtBeginning(
+          createVariable.get.origin,
+          createVariable.get.names(0).asInstanceOf[UnresolvedIdentifier].nameParts)
+
       case (State.STATEMENT, State.VARIABLE) =>
         throw SqlScriptingErrors.variableDeclarationOnlyAtBeginning(
           createVariable.get.origin,
@@ -238,6 +272,11 @@ class CompoundBodyParsingContext {
           createVariable.get.names(0).asInstanceOf[UnresolvedIdentifier].nameParts)
 
       // Invalid transitions to CONDITION state.
+      case (State.CURSOR, State.CONDITION) =>
+        throw SqlScriptingErrors.conditionDeclarationNotAtStartOfCompound(
+          CurrentOrigin.get,
+          errorCondition.get.conditionName)
+
       case (State.STATEMENT, State.CONDITION) =>
         throw SqlScriptingErrors.conditionDeclarationNotAtStartOfCompound(
           CurrentOrigin.get,
@@ -248,6 +287,13 @@ class CompoundBodyParsingContext {
           createVariable.get.origin,
           createVariable.get.names(0).asInstanceOf[UnresolvedIdentifier].nameParts)
 
+      // Invalid transitions to CURSOR state.
+      case (State.STATEMENT, State.CURSOR) =>
+        throw SqlScriptingErrors.cursorDeclarationNotAtStartOfCompound(CurrentOrigin.get)
+
+      case (State.HANDLER, State.CURSOR) =>
+        throw SqlScriptingErrors.cursorDeclarationNotAtStartOfCompound(CurrentOrigin.get)
+
       // Invalid transitions to HANDLER state.
       case (State.STATEMENT, State.HANDLER) =>
         throw SqlScriptingErrors.handlerDeclarationInWrongPlace(CurrentOrigin.get)
@@ -279,34 +325,49 @@ class SqlScriptingLabelContext {
    * @param beginLabelCtx Begin label context.
    * @param endLabelCtx The end label context.
    */
+  /**
+   * Get label text from label context, handling IDENTIFIER() syntax.
+   */
+  private def getLabelText(ctx: ParserRuleContext): String = {
+    val astBuilder = new DataTypeAstBuilder {
+      override protected def parseMultipartIdentifier(identifier: String): Seq[String] = {
+        CatalystSqlParser.parseMultipartIdentifier(identifier)
+      }
+    }
+    val parts = astBuilder.extractIdentifierParts(ctx)
+    if (parts.size > 1) {
+      throw new ParseException(
+        errorClass = "IDENTIFIER_TOO_MANY_NAME_PARTS",
+        messageParameters = Map("identifier" -> parts.map(part => s"`$part`").mkString("."),
+          "limit" -> "1"),
+        ctx)
+    }
+    parts.head
+  }
+
   private def checkLabels(
       beginLabelCtx: Option[BeginLabelContext],
-      endLabelCtx: Option[EndLabelContext]) : Unit = {
+      endLabelCtx: Option[EndLabelContext]): Unit = {
+    // Check label matching and other constraints.
     (beginLabelCtx, endLabelCtx) match {
       // Throw an error if labels do not match.
-      case (Some(bl: BeginLabelContext), Some(el: EndLabelContext))
-        if bl.multipartIdentifier().getText.toLowerCase(Locale.ROOT) !=
-            el.multipartIdentifier().getText.toLowerCase(Locale.ROOT) =>
-        withOrigin(bl) {
-          throw SqlScriptingErrors.labelsMismatch(
-            CurrentOrigin.get,
-            bl.multipartIdentifier().getText,
-            el.multipartIdentifier().getText)
-        }
-      // Throw an error if label is qualified.
-      case (Some(bl: BeginLabelContext), _)
-        if bl.multipartIdentifier().parts.size() > 1 =>
-        withOrigin(bl) {
-          throw SqlScriptingErrors.labelCannotBeQualified(
-            CurrentOrigin.get,
-            bl.multipartIdentifier().getText.toLowerCase(Locale.ROOT)
-          )
+      case (Some(bl: BeginLabelContext), Some(el: EndLabelContext)) =>
+        val beginLabel = getLabelText(bl.strictIdentifier()).toLowerCase(Locale.ROOT)
+        val endLabel = getLabelText(el.strictIdentifier()).toLowerCase(Locale.ROOT)
+        if (beginLabel != endLabel) {
+          withOrigin(bl) {
+            throw SqlScriptingErrors.labelsMismatch(
+              CurrentOrigin.get,
+              getLabelText(bl.strictIdentifier()),
+              getLabelText(el.strictIdentifier()))
+          }
         }
       // Throw an error if end label exists without begin label.
       case (None, Some(el: EndLabelContext)) =>
         withOrigin(el) {
           throw SqlScriptingErrors.endLabelWithoutBeginLabel(
-            CurrentOrigin.get, el.multipartIdentifier().getText)
+            CurrentOrigin.get,
+            getLabelText(el.strictIdentifier()))
         }
       case _ =>
     }
@@ -314,7 +375,7 @@ class SqlScriptingLabelContext {
 
   /** Check if the label is defined. */
   private def isLabelDefined(beginLabelCtx: Option[BeginLabelContext]): Boolean = {
-    beginLabelCtx.map(_.multipartIdentifier().getText).isDefined
+    beginLabelCtx.isDefined
   }
 
   /**
@@ -322,13 +383,13 @@ class SqlScriptingLabelContext {
    * If the identifier is contained within seenLabels, raise an exception.
    */
   private def assertIdentifierNotInSeenLabels(
-      identifierCtx: Option[MultipartIdentifierContext]): Unit = {
+      identifierCtx: Option[StrictIdentifierContext]): Unit = {
     identifierCtx.foreach { ctx =>
-      val identifierName = ctx.getText
-      if (seenLabels.contains(identifierName.toLowerCase(Locale.ROOT))) {
+      val identifierName = getLabelText(ctx).toLowerCase(Locale.ROOT)
+      if (seenLabels.contains(identifierName)) {
         withOrigin(ctx) {
           throw SqlScriptingErrors
-            .duplicateLabels(CurrentOrigin.get, identifierName.toLowerCase(Locale.ROOT))
+            .duplicateLabels(CurrentOrigin.get, identifierName)
         }
       }
     }
@@ -348,7 +409,7 @@ class SqlScriptingLabelContext {
 
     // Get label text and add it to seenLabels.
     val labelText = if (isLabelDefined(beginLabelCtx)) {
-      val txt = beginLabelCtx.get.multipartIdentifier().getText.toLowerCase(Locale.ROOT)
+      val txt = getLabelText(beginLabelCtx.get.strictIdentifier()).toLowerCase(Locale.ROOT)
       if (seenLabels.contains(txt)) {
         withOrigin(beginLabelCtx.get) {
           throw SqlScriptingErrors.duplicateLabels(CurrentOrigin.get, txt)
@@ -374,18 +435,18 @@ class SqlScriptingLabelContext {
    */
   def exitLabeledScope(beginLabelCtx: Option[BeginLabelContext]): Unit = {
     if (isLabelDefined(beginLabelCtx)) {
-      seenLabels.remove(beginLabelCtx.get.multipartIdentifier().getText.toLowerCase(Locale.ROOT))
+      seenLabels.remove(getLabelText(beginLabelCtx.get.strictIdentifier()).toLowerCase(Locale.ROOT))
     }
   }
 
   /**
    * Enter a for loop scope.
-   * If the for loop variable is defined, it will be asserted to not be inside seenLabels;
+   * If the for loop variable is defined, it will be asserted to not be inside seenLabels.
    * Then, if the for loop variable is defined, it will be added to seenLabels.
    */
-  def enterForScope(identifierCtx: Option[MultipartIdentifierContext]): Unit = {
+  def enterForScope(identifierCtx: Option[StrictIdentifierContext]): Unit = {
     identifierCtx.foreach { ctx =>
-      val identifierName = ctx.getText
+      val identifierName = getLabelText(ctx)
       assertIdentifierNotInSeenLabels(identifierCtx)
       seenLabels.add(identifierName.toLowerCase(Locale.ROOT))
 
@@ -403,9 +464,9 @@ class SqlScriptingLabelContext {
    * Exit a for loop scope.
    * If the for loop variable is defined, it will be removed from seenLabels.
    */
-  def exitForScope(identifierCtx: Option[MultipartIdentifierContext]): Unit = {
+  def exitForScope(identifierCtx: Option[StrictIdentifierContext]): Unit = {
     identifierCtx.foreach { ctx =>
-      val identifierName = ctx.getText
+      val identifierName = getLabelText(ctx)
       seenLabels.remove(identifierName.toLowerCase(Locale.ROOT))
     }
   }
