@@ -21,11 +21,11 @@ import scala.annotation.nowarn
 
 import org.apache.spark.SparkThrowable
 import org.apache.spark.sql.catalyst.{FunctionIdentifier, TableIdentifier}
-import org.apache.spark.sql.catalyst.analysis.{AnalysisTest, NamedParameter, PosParameter, RelationTimeTravel, UnresolvedAlias, UnresolvedAttribute, UnresolvedFunction, UnresolvedGenerator, UnresolvedInlineTable, UnresolvedRelation, UnresolvedStar, UnresolvedSubqueryColumnAliases, UnresolvedTableValuedFunction, UnresolvedTVFAliases}
+import org.apache.spark.sql.catalyst.analysis.{AnalysisTest, RelationTimeTravel, UnresolvedAlias, UnresolvedAttribute, UnresolvedFunction, UnresolvedGenerator, UnresolvedInlineTable, UnresolvedRelation, UnresolvedStar, UnresolvedSubqueryColumnAliases, UnresolvedTableValuedFunction, UnresolvedTVFAliases}
 import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.catalyst.plans._
 import org.apache.spark.sql.catalyst.plans.logical._
-import org.apache.spark.sql.catalyst.util.EvaluateUnresolvedInlineTable
+import org.apache.spark.sql.catalyst.util.{EvaluateUnresolvedInlineTable, IntervalUtils}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types.{Decimal, DecimalType, IntegerType, LongType, StringType}
 import org.apache.spark.sql.util.CaseInsensitiveStringMap
@@ -1395,7 +1395,7 @@ class PlanParserSuite extends AnalysisTest {
     checkError(
       exception = parseException(sql2),
       condition = "PARSE_SYNTAX_ERROR",
-      parameters = Map("error" -> "'IN'", "hint" -> ""))
+      parameters = Map("error" -> "'INTO'", "hint" -> ""))
   }
 
   test("relation in v2 catalog") {
@@ -1908,28 +1908,50 @@ class PlanParserSuite extends AnalysisTest {
     )
   }
 
-  test("SPARK-41271: parsing of named parameters") {
+  test("SPARK-41271: parameter substitution before parsing") {
+    // Test successful parameter substitution
+    val substitutor = new org.apache.spark.sql.catalyst.parser.SubstituteParamsParser()
+
+    // Test named parameters
+    val sql1 = "SELECT :param_1"
+    val (substituted1, _, _) = substitutor.substitute(
+      sql1, Map("param_1" -> "42"))
     comparePlans(
-      parsePlan("SELECT :param_1"),
-      Project(UnresolvedAlias(NamedParameter("param_1"), None) :: Nil, OneRowRelation()))
+      parsePlan(substituted1),
+      Project(UnresolvedAlias(Literal(42), None) :: Nil, OneRowRelation()))
+
+    val sql2 = "SELECT abs(:1Abc)"
+    val (substituted2, _, _) = substitutor.substitute(
+      sql2, Map("1Abc" -> "123"))
     comparePlans(
-      parsePlan("SELECT abs(:1Abc)"),
+      parsePlan(substituted2),
       Project(UnresolvedAlias(
         UnresolvedFunction(
           "abs" :: Nil,
-          NamedParameter("1Abc") :: Nil,
+          Literal(123) :: Nil,
           isDistinct = false), None) :: Nil,
         OneRowRelation()))
+
+    val sql3 = "SELECT * FROM a LIMIT :limitA"
+    val (substituted3, _, _) = substitutor.substitute(
+      sql3, Map("limitA" -> "10"))
     comparePlans(
-      parsePlan("SELECT * FROM a LIMIT :limitA"),
-      table("a").select(star()).limit(NamedParameter("limitA")))
-    // Invalid empty name and invalid symbol in a name
+      parsePlan(substituted3),
+      table("a").select(star()).limit(Literal(10)))
+
+    // Test error cases - invalid parameter syntax should be caught by SubstituteParamsParser
     checkError(
-      exception = parseException(s"SELECT :-"),
+      exception = intercept[org.apache.spark.sql.catalyst.parser.ParseException] {
+        val (substituted, _, _) = substitutor.substitute("SELECT :-")
+        substituted
+      },
       condition = "PARSE_SYNTAX_ERROR",
       parameters = Map("error" -> "'-'", "hint" -> ""))
     checkError(
-      exception = parseException(s"SELECT :"),
+      exception = intercept[org.apache.spark.sql.catalyst.parser.ParseException] {
+        val (substituted, _, _) = substitutor.substitute("SELECT :")
+        substituted
+      },
       condition = "PARSE_SYNTAX_ERROR",
       parameters = Map("error" -> "end of input", "hint" -> ""))
   }
@@ -1953,21 +1975,36 @@ class PlanParserSuite extends AnalysisTest {
     }
   }
 
-  test("SPARK-44066: parsing of positional parameters") {
+  test("SPARK-44066: positional parameter substitution before parsing") {
+    // Test successful positional parameter substitution
+    val substitutor = new org.apache.spark.sql.catalyst.parser.SubstituteParamsParser()
+
+    // Test positional parameters
+    val sql1 = "SELECT ?"
+    val (substituted1, _, _) = substitutor.substitute(
+      sql1, positionalParams = List("42"))
     comparePlans(
-      parsePlan("SELECT ?"),
-      Project(UnresolvedAlias(PosParameter(7), None) :: Nil, OneRowRelation()))
+      parsePlan(substituted1),
+      Project(UnresolvedAlias(Literal(42), None) :: Nil, OneRowRelation()))
+
+    val sql2 = "SELECT abs(?)"
+    val (substituted2, _, _) = substitutor.substitute(
+      sql2, positionalParams = List("123"))
     comparePlans(
-      parsePlan("SELECT abs(?)"),
+      parsePlan(substituted2),
       Project(UnresolvedAlias(
         UnresolvedFunction(
           "abs" :: Nil,
-          PosParameter(11) :: Nil,
+          Literal(123) :: Nil,
           isDistinct = false), None) :: Nil,
         OneRowRelation()))
+
+    val sql3 = "SELECT * FROM a LIMIT ?"
+    val (substituted3, _, _) = substitutor.substitute(
+      sql3, positionalParams = List("10"))
     comparePlans(
-      parsePlan("SELECT * FROM a LIMIT ?"),
-      table("a").select(star()).limit(PosParameter(22)))
+      parsePlan(substituted3),
+      table("a").select(star()).limit(Literal(10)))
   }
 
   test("SPARK-45189: Creating UnresolvedRelation from TableIdentifier should include the" +
@@ -1980,4 +2017,150 @@ class PlanParserSuite extends AnalysisTest {
     assert(unresolvedRelation2.options == CaseInsensitiveStringMap.empty)
     assert(unresolvedRelation2.isStreaming)
   }
+
+  test("watermark clause - table & attribute reference") {
+    assertEqual(
+      """
+        |SELECT *
+        |FROM testData
+        |WATERMARK ts DELAY OF INTERVAL 10 seconds AS tbl
+        |WHERE a > 1
+        |""".stripMargin,
+      table("testData")
+        .as("tbl")
+        .unresolvedWithWatermark(
+          UnresolvedAttribute("ts"),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .where($"a" > 1)
+        .select(UnresolvedStar(None))
+    )
+  }
+
+  test("watermark clause - table & expression with alias") {
+    assertEqual(
+      """
+        |SELECT *
+        |FROM testData
+        |WATERMARK timestamp_seconds(value) AS eventTime DELAY OF INTERVAL 10 seconds AS tbl
+        |WHERE a > 1
+        |""".stripMargin,
+      table("testData")
+        .as("tbl")
+        .unresolvedWithWatermark(
+          Alias(
+            UnresolvedFunction(
+              Seq("timestamp_seconds"), Seq(UnresolvedAttribute("value")), isDistinct = false),
+            "eventTime")(),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .where($"a" > 1)
+        .select(UnresolvedStar(None))
+    )
+  }
+
+  test("watermark clause - table & expression without alias") {
+    assertEqual(
+      """
+        |SELECT *
+        |FROM testData
+        |WATERMARK timestamp_seconds(value) DELAY OF INTERVAL 10 seconds AS tbl
+        |WHERE a > 1
+        |""".stripMargin,
+      table("testData")
+        .as("tbl")
+        .unresolvedWithWatermark(
+          UnresolvedAlias(
+            UnresolvedFunction(
+              Seq("timestamp_seconds"), Seq(UnresolvedAttribute("value")), isDistinct = false)),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .where($"a" > 1)
+        .select(UnresolvedStar(None))
+    )
+  }
+
+  test("watermark clause - aliased query") {
+    assertEqual(
+      """
+        |SELECT *
+        |FROM
+        |(
+        |    SELECT *
+        |    FROM testData
+        |)
+        |WATERMARK ts DELAY OF INTERVAL 10 seconds AS tbl
+        |WHERE a > 1
+        |""".stripMargin,
+      table("testData")
+        .select(UnresolvedStar(None))
+        .as("tbl")
+        .unresolvedWithWatermark(
+          UnresolvedAttribute("ts"),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .where($"a" > 1)
+        .select(UnresolvedStar(None))
+    )
+  }
+
+  test("watermark clause - subquery") {
+    assertEqual(
+      """
+        |SELECT key, time
+        |FROM
+        |(
+        |    SELECT key, time
+        |    FROM
+        |    testData
+        |    WATERMARK timestamp_seconds(ts) AS time DELAY OF INTERVAL 10 seconds
+        |)
+        |AS tbl
+        |WHERE key = 'a'
+        |""".stripMargin,
+      table("testData")
+        .unresolvedWithWatermark(
+          Alias(
+            UnresolvedFunction(
+              Seq("timestamp_seconds"), Seq(UnresolvedAttribute("ts")), isDistinct = false),
+            "time")(),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .select($"key", $"time")
+        .as("tbl")
+        .where($"key" === "a")
+        .select($"key", $"time")
+    )
+  }
+
+  test("watermark clause - table valued function") {
+    assertEqual(
+      """
+        |SELECT *
+        |FROM
+        |mock_tvf(1, 'a')
+        |WATERMARK ts DELAY OF INTERVAL 10 seconds AS dst
+        |WHERE a > 1
+        |""".stripMargin,
+      UnresolvedTableValuedFunction("mock_tvf", Seq(Literal(1), Literal("a")))
+        .unresolvedWithWatermark(
+          UnresolvedAttribute("ts"),
+          IntervalUtils.fromIntervalString("INTERVAL 10 seconds"))
+        .as("dst")
+        .where($"a" > 1)
+        .select(UnresolvedStar(None))
+    )
+  }
+
+  test("watermark clause - inline table (not allowed)") {
+    val query = """
+        |SELECT *
+        |FROM
+        |VALUES (1, 1), (2, 2)
+        |WATERMARK ts DELAY OF INTERVAL 10 seconds AS dst
+        |WHERE a > 1
+        |""".stripMargin
+    checkError(
+      exception = parseException(query),
+      condition = "PARSE_SYNTAX_ERROR",
+      parameters = Map("error" -> "'ts'", "hint" -> ""))
+  }
+
+  private def intercept(sqlCommand: String, messages: String*): Unit =
+    interceptParseException(parsePlan)(sqlCommand, messages: _*)()
 }
