@@ -16,6 +16,7 @@
  */
 package org.apache.spark.sql.connect.planner
 
+import java.io.SequenceInputStream
 import java.util.UUID
 import java.util.concurrent.Semaphore
 
@@ -30,7 +31,6 @@ import org.apache.arrow.memory.RootAllocator
 import org.apache.arrow.vector.{BigIntVector, Float8Vector}
 import org.apache.arrow.vector.ipc.ArrowStreamReader
 import org.mockito.Mockito.when
-import org.scalatest.Tag
 import org.scalatest.concurrent.Eventually
 import org.scalatest.time.SpanSugar.convertIntToGrainOfTime
 import org.scalatestplus.mockito.MockitoSugar
@@ -48,7 +48,7 @@ import org.apache.spark.sql.connect.dsl.MockRemoteSession
 import org.apache.spark.sql.connect.dsl.expressions._
 import org.apache.spark.sql.connect.dsl.plans._
 import org.apache.spark.sql.connect.plugin.SparkConnectPluginRegistry
-import org.apache.spark.sql.connect.service.{ExecuteHolder, ExecuteStatus, SessionStatus, SparkConnectAnalyzeHandler, SparkConnectService, SparkListenerConnectOperationStarted}
+import org.apache.spark.sql.connect.service.{ExecuteHolder, ExecuteKey, ExecuteStatus, SessionStatus, SparkConnectAnalyzeHandler, SparkConnectService, SparkListenerConnectOperationStarted}
 import org.apache.spark.sql.connector.catalog.InMemoryPartitionTableCatalog
 import org.apache.spark.sql.streaming.StreamingQuery
 import org.apache.spark.sql.test.SharedSparkSession
@@ -65,6 +65,12 @@ class SparkConnectServiceSuite
     with Logging
     with SparkConnectPlanTest {
 
+  override def beforeEach(): Unit = {
+    super.beforeEach()
+    SparkConnectService.sessionManager.invalidateAllSessions()
+    SparkConnectService.sessionManager.initializeBaseSession(() => spark.newSession())
+  }
+
   private def sparkSessionHolder = SparkConnectTestUtils.createDummySessionHolder(spark)
   private def DEFAULT_UUID = UUID.fromString("89ea6117-1f45-4c03-ae27-f47c6aded093")
 
@@ -193,10 +199,11 @@ class SparkConnectServiceSuite
           }
 
           override def onCompleted(): Unit = {
+            verifyEvents.onCompleted(Some(100))
             done = true
           }
         })
-      verifyEvents.onCompleted(Some(100))
+      verifyEvents.assertClosed()
       // The current implementation is expected to be blocking. This is here to make sure it is.
       assert(done)
 
@@ -294,10 +301,11 @@ class SparkConnectServiceSuite
           }
 
           override def onCompleted(): Unit = {
+            verifyEvents.onCompleted(Some(6))
             done = true
           }
         })
-      verifyEvents.onCompleted(Some(6))
+      verifyEvents.assertClosed()
       // The current implementation is expected to be blocking. This is here to make sure it is.
       assert(done)
 
@@ -375,6 +383,153 @@ class SparkConnectServiceSuite
     }
   }
 
+  Seq((Some(1024), None), (None, Some(1024))).foreach {
+    // Two cases are tested here:
+    // (a) client preferred chunk size is set: the server should respect it
+    // (b) client preferred chunk size is not set: the server should use its own max chunk size
+    case (preferredChunkSizeOption, maxChunkSizeOption) =>
+      test(
+        s"Arrow batch chunking with preferredMaxChunkSize=$preferredChunkSizeOption, " +
+          s"maxChunkSize=$maxChunkSizeOption") {
+        withEvents { verifyEvents =>
+          val maxChunkSizeConfigValue = maxChunkSizeOption match {
+            case Some(maxChunkSize) => maxChunkSize.toString
+            case None => Connect.CONNECT_SESSION_RESULT_CHUNKING_MAX_CHUNK_SIZE.defaultValueString
+          }
+          withSparkConf(
+            Connect.CONNECT_SESSION_RESULT_CHUNKING_MAX_CHUNK_SIZE.key ->
+              maxChunkSizeConfigValue) {
+            val instance = new SparkConnectService(false)
+            val connect = new MockRemoteSession()
+            val context = proto.UserContext
+              .newBuilder()
+              .setUserId("c1")
+              .build()
+            val plan = proto.Plan
+              .newBuilder()
+              .setRoot(
+                connect.sql("select id, CAST(id + 0.5 AS DOUBLE) from range(0, 2000, 1, 4)"))
+              .build()
+            val request = proto.ExecutePlanRequest
+              .newBuilder()
+              .setPlan(plan)
+              .addRequestOptions(
+                proto.ExecutePlanRequest.RequestOption
+                  .newBuilder()
+                  .setResultChunkingOptions({
+                    val builder = proto.ResultChunkingOptions
+                      .newBuilder()
+                      .setAllowArrowBatchChunking(true)
+                    preferredChunkSizeOption.foreach { size =>
+                      builder.setPreferredArrowChunkSize(size.toLong)
+                    }
+                    builder.build()
+                  })
+                  .build())
+              .setUserContext(context)
+              .setSessionId(UUID.randomUUID.toString())
+              .build()
+
+            // Execute plan.
+            var done = false
+            val responses = mutable.Buffer.empty[proto.ExecutePlanResponse]
+            instance.executePlan(
+              request,
+              new StreamObserver[proto.ExecutePlanResponse] {
+                override def onNext(v: proto.ExecutePlanResponse): Unit = {
+                  responses += v
+                  verifyEvents.onNext(v)
+                }
+
+                override def onError(throwable: Throwable): Unit = {
+                  verifyEvents.onError(throwable)
+                  throw throwable
+                }
+
+                override def onCompleted(): Unit = {
+                  done = true
+                }
+              })
+            verifyEvents.onCompleted(Some(2000))
+            // The current implementation is expected to be blocking. This is here to make sure it
+            // is.
+            assert(done)
+
+            // ArrowBatch chunks of 4 partitions + Metrics + optional progress messages
+            val filteredResponses = responses.filter(!_.hasExecutionProgress)
+
+            // Make sure the first response is schema only
+            val head = filteredResponses.head
+            assert(head.hasSchema && !head.hasArrowBatch && !head.hasMetrics)
+
+            // Make sure the last response is metrics only
+            val last = filteredResponses.last
+            assert(last.hasMetrics && !last.hasSchema && !last.hasArrowBatch)
+
+            val allocator = new RootAllocator()
+
+            // Check the 'data' batches
+            val arrowResponseQueue = mutable.Queue(filteredResponses.tail.dropRight(1).toSeq: _*)
+
+            def dequeueNextBatch(): List[proto.ExecutePlanResponse] = {
+              if (arrowResponseQueue.isEmpty) return Nil
+              val n = arrowResponseQueue.front.getArrowBatch.getNumChunksInBatch.toInt
+              List.fill(n)(arrowResponseQueue.dequeue())
+            }
+
+            var batchCount = 0
+            var expectedId = 0L
+            var previousEId = 0.0d
+            while (arrowResponseQueue.nonEmpty) {
+              val batch = dequeueNextBatch()
+              // In this example, the max chunk size is set to a small value, so each Arrow batch
+              // should be split into multiple chunks.
+              assert(batch.size > 5)
+              batchCount += 1
+
+              val rowCount = batch.head.getArrowBatch.getRowCount
+              val rowStartOffset = batch.head.getArrowBatch.getStartOffset
+              batch.zipWithIndex.foreach { case (chunk, chunkIndex) =>
+                assert(chunk.getArrowBatch.getChunkIndex == chunkIndex)
+                assert(chunk.getArrowBatch.getNumChunksInBatch == batch.size)
+                assert(chunk.getArrowBatch.getRowCount == rowCount)
+                assert(chunk.getArrowBatch.getStartOffset == rowStartOffset)
+                assert(chunk.getArrowBatch.getData != null)
+                assert(chunk.getArrowBatch.getData.size() > 0)
+                assert(
+                  chunk.getArrowBatch.getData.size() <= preferredChunkSizeOption.getOrElse(
+                    maxChunkSizeOption.get))
+              }
+
+              // Reassemble the chunks into a single Arrow batch and validate its content.
+              val input = new SequenceInputStream(
+                batch.iterator.map(_.getArrowBatch.getData.newInput()).asJavaEnumeration)
+              val reader = new ArrowStreamReader(input, allocator)
+              while (reader.loadNextBatch()) {
+                val root = reader.getVectorSchemaRoot
+                val idVector = root.getVector(0).asInstanceOf[BigIntVector]
+                val eidVector = root.getVector(1).asInstanceOf[Float8Vector]
+                val numRows = root.getRowCount
+                var i = 0
+                while (i < numRows) {
+                  assert(idVector.get(i) == expectedId)
+                  expectedId += 1
+                  val eid = eidVector.get(i)
+                  assert(eid > previousEId)
+                  previousEId = eid
+                  i += 1
+                }
+              }
+              reader.close()
+            }
+            // There are 4 batches (partitions) in total.
+            assert(batchCount == 4)
+            allocator.close()
+          }
+        }
+      }
+  }
+
   gridTest("SPARK-43923: commands send events")(
     Seq(
       (
@@ -530,10 +685,11 @@ class SparkConnectServiceSuite
           }
 
           override def onCompleted(): Unit = {
+            verifyEvents.onCompleted(producedNumRows)
             done = true
           }
         })
-      verifyEvents.onCompleted(producedNumRows)
+      verifyEvents.assertClosed()
       // The current implementation is expected to be blocking.
       // This is here to make sure it is.
       assert(done)
@@ -621,7 +777,7 @@ class SparkConnectServiceSuite
           }
         })
       thread.join()
-      verifyEvents.onCompleted()
+      verifyEvents.assertClosed()
     }
   }
 
@@ -684,7 +840,43 @@ class SparkConnectServiceSuite
           }
         })
       assert(failures.isEmpty, s"this should have no failures but got $failures")
-      verifyEvents.onCompleted()
+      verifyEvents.assertClosed()
+    }
+  }
+
+  test("SPARK-51818: AnalyzePlanRequest does not execute the command") {
+    withTable("test") {
+      spark.sql("""
+                  | CREATE TABLE test (col1 INT, col2 STRING)
+                  |""".stripMargin)
+      val sqlString = "DROP TABLE test"
+      val plan = proto.Plan
+        .newBuilder()
+        .setRoot(
+          proto.Relation
+            .newBuilder()
+            .setCommon(proto.RelationCommon.newBuilder().setPlanId(1))
+            .setSql(proto.SQL.newBuilder().setQuery(sqlString).build())
+            .build())
+        .build()
+
+      val handler = new SparkConnectAnalyzeHandler(null)
+
+      val request = proto.AnalyzePlanRequest
+        .newBuilder()
+        .setExplain(
+          proto.AnalyzePlanRequest.Explain
+            .newBuilder()
+            .setPlan(plan)
+            .setExplainMode(proto.AnalyzePlanRequest.Explain.ExplainMode.EXPLAIN_MODE_EXTENDED)
+            .build())
+        .build()
+
+      handler.process(request, sparkSessionHolder)
+
+      // assert that table was not dropped
+      val tableExists = spark.catalog.tableExists("test")
+      assert(tableExists, "Table test should still exist after analyze request of DROP TABLE")
     }
   }
 
@@ -861,38 +1053,42 @@ class SparkConnectServiceSuite
     }
   }
 
-  protected def gridTest[A](testNamePrefix: String, testTags: Tag*)(params: Seq[A])(
-      testFun: A => Unit): Unit = {
-    for (param <- params) {
-      test(testNamePrefix + s" ($param)", testTags: _*)(testFun(param))
-    }
-  }
-
   class VerifyEvents(val sparkContext: SparkContext) {
     val listener: MockSparkListener = new MockSparkListener()
     val listenerBus = sparkContext.listenerBus
+    val EVENT_WAIT_TIMEOUT = timeout(10.seconds)
     val LISTENER_BUS_TIMEOUT = 30000
     def executeHolder: ExecuteHolder = {
-      assert(listener.executeHolder.isDefined)
-      listener.executeHolder.get
+      // An ExecuteHolder shall be set eventually through MockSparkListener
+      Eventually.eventually(EVENT_WAIT_TIMEOUT) {
+        assert(
+          listener.executeHolder.isDefined,
+          s"No events have been posted in $EVENT_WAIT_TIMEOUT")
+        listener.executeHolder.get
+      }
     }
     def onNext(v: proto.ExecutePlanResponse): Unit = {
-      if (v.hasSchema) {
-        assert(executeHolder.eventsManager.status == ExecuteStatus.Analyzed)
-      }
       if (v.hasMetrics) {
         assert(executeHolder.eventsManager.status == ExecuteStatus.Finished)
       }
     }
     def onError(throwable: Throwable): Unit = {
       assert(executeHolder.eventsManager.hasCanceled.isEmpty)
-      assert(executeHolder.eventsManager.hasError.isDefined)
+      Eventually.eventually(EVENT_WAIT_TIMEOUT) {
+        assert(
+          executeHolder.eventsManager.hasError.isDefined,
+          s"Error has not been recorded in events manager within $EVENT_WAIT_TIMEOUT")
+      }
     }
     def onCompleted(producedRowCount: Option[Long] = None): Unit = {
       assert(executeHolder.eventsManager.getProducedRowCount == producedRowCount)
+    }
+    def assertClosed(): Unit = {
       // The eventsManager is closed asynchronously
-      Eventually.eventually(timeout(1.seconds)) {
-        assert(executeHolder.eventsManager.status == ExecuteStatus.Closed)
+      Eventually.eventually(EVENT_WAIT_TIMEOUT) {
+        assert(
+          executeHolder.eventsManager.status == ExecuteStatus.Closed,
+          s"Execution has not been completed in $EVENT_WAIT_TIMEOUT")
       }
     }
     def onCanceled(): Unit = {
@@ -911,14 +1107,17 @@ class SparkConnectServiceSuite
   }
   class MockSparkListener() extends SparkListener {
     val semaphoreStarted = new Semaphore(0)
-    var executeHolder = Option.empty[ExecuteHolder]
+    // Accessed by multiple threads in parallel.
+    @volatile var executeHolder = Option.empty[ExecuteHolder]
     override def onOtherEvent(event: SparkListenerEvent): Unit = {
       event match {
         case e: SparkListenerConnectOperationStarted =>
           semaphoreStarted.release()
           val sessionHolder =
             SparkConnectService.getOrCreateIsolatedSession(e.userId, e.sessionId, None)
-          executeHolder = sessionHolder.executeHolder(e.operationId)
+          val executeKey =
+            ExecuteKey(sessionHolder.userId, sessionHolder.sessionId, e.operationId)
+          executeHolder = SparkConnectService.executionManager.getExecuteHolder(executeKey)
         case _ =>
       }
     }
