@@ -267,6 +267,8 @@ private object RowToColumnConverter {
       case LongType | TimestampType | TimestampNTZType | _: DayTimeIntervalType => LongConverter
       case DoubleType => DoubleConverter
       case StringType => StringConverter
+      case _: GeographyType => GeographyConverter
+      case _: GeometryType => GeometryConverter
       case CalendarIntervalType => CalendarConverter
       case VariantType => VariantConverter
       case at: ArrayType => ArrayConverter(getConverterForType(at.elementType, at.containsNull))
@@ -338,6 +340,20 @@ private object RowToColumnConverter {
     }
   }
 
+  private object GeographyConverter extends TypeConverter {
+    override def append(row: SpecializedGetters, column: Int, cv: WritableColumnVector): Unit = {
+      val data = row.getGeography(column).getBytes
+      cv.appendByteArray(data, 0, data.length)
+    }
+  }
+
+  private object GeometryConverter extends TypeConverter {
+    override def append(row: SpecializedGetters, column: Int, cv: WritableColumnVector): Unit = {
+      val data = row.getGeometry(column).getBytes
+      cv.appendByteArray(data, 0, data.length)
+    }
+  }
+
   private object CalendarConverter extends TypeConverter {
     override def append(row: SpecializedGetters, column: Int, cv: WritableColumnVector): Unit = {
       val c = row.getInterval(column)
