@@ -19,34 +19,37 @@ package org.apache.spark.sql.execution.streaming.runtime
 
 import scala.collection.mutable.{Map => MutableMap}
 import scala.collection.mutable
+import scala.jdk.CollectionConverters._
 import scala.util.control.NonFatal
 
 import org.apache.hadoop.fs.Path
 
+import org.apache.spark.{SparkIllegalArgumentException, SparkIllegalStateException}
 import org.apache.spark.internal.LogKeys
-import org.apache.spark.internal.LogKeys.BATCH_ID
+import org.apache.spark.internal.LogKeys._
 import org.apache.spark.sql.catalyst.encoders.ExpressionEncoder
 import org.apache.spark.sql.catalyst.expressions.{Alias, Attribute, CurrentBatchTimestamp, CurrentDate, CurrentTimestamp, FileSourceMetadataAttribute, LocalTimestamp}
-import org.apache.spark.sql.catalyst.plans.logical.{Aggregate, GlobalLimit, LeafNode, LocalRelation, LogicalPlan, Project, StreamSourceAwareLogicalPlan}
+import org.apache.spark.sql.catalyst.plans.logical.{Aggregate, Deduplicate, DeduplicateWithinWatermark, Distinct, FlatMapGroupsInPandasWithState, FlatMapGroupsWithState, GlobalLimit, Join, LeafNode, LocalRelation, LogicalPlan, Project, StreamSourceAwareLogicalPlan, TransformWithState, TransformWithStateInPySpark}
 import org.apache.spark.sql.catalyst.streaming.{StreamingRelationV2, WriteToStream}
 import org.apache.spark.sql.catalyst.trees.TreePattern.CURRENT_LIKE
 import org.apache.spark.sql.catalyst.util.truncatedString
 import org.apache.spark.sql.classic.{Dataset, SparkSession}
 import org.apache.spark.sql.classic.ClassicConversions.castToImpl
 import org.apache.spark.sql.connector.catalog.{SupportsRead, SupportsWrite, TableCapability}
-import org.apache.spark.sql.connector.read.streaming.{MicroBatchStream, Offset => OffsetV2, ReadLimit, SparkDataStream, SupportsAdmissionControl, SupportsTriggerAvailableNow}
+import org.apache.spark.sql.connector.read.streaming.{MicroBatchStream, Offset => OffsetV2, ReadLimit, SparkDataStream, SupportsAdmissionControl, SupportsRealTimeMode, SupportsTriggerAvailableNow}
 import org.apache.spark.sql.errors.QueryExecutionErrors
 import org.apache.spark.sql.execution.{SparkPlan, SQLExecution}
 import org.apache.spark.sql.execution.datasources.LogicalRelation
-import org.apache.spark.sql.execution.datasources.v2.{DataSourceV2Relation, StreamingDataSourceV2Relation, StreamingDataSourceV2ScanRelation, StreamWriterCommitProgress, WriteToDataSourceV2Exec}
-import org.apache.spark.sql.execution.streaming.{AvailableNowTrigger, Offset, OneTimeTrigger, ProcessingTimeTrigger, Sink, Source}
-import org.apache.spark.sql.execution.streaming.checkpointing.{CheckpointFileManager, CommitMetadata, OffsetSeq, OffsetSeqMetadata}
+import org.apache.spark.sql.execution.datasources.v2.{DataSourceV2Relation, RealTimeStreamScanExec, StreamingDataSourceV2Relation, StreamingDataSourceV2ScanRelation, StreamWriterCommitProgress, WriteToDataSourceV2Exec}
+import org.apache.spark.sql.execution.streaming.{AvailableNowTrigger, Offset, OneTimeTrigger, ProcessingTimeTrigger, RealTimeModeAllowlist, RealTimeTrigger, Sink, Source, StreamingQueryPlanTraverseHelper}
+import org.apache.spark.sql.execution.streaming.checkpointing.{CheckpointFileManager, CommitMetadata, OffsetSeqBase, OffsetSeqLog, OffsetSeqMetadata, OffsetSeqMetadataV2}
 import org.apache.spark.sql.execution.streaming.operators.stateful.{StatefulOperatorStateInfo, StatefulOpStateStoreCheckpointInfo, StateStoreWriter}
 import org.apache.spark.sql.execution.streaming.runtime.AcceptsLatestSeenOffsetHandler
 import org.apache.spark.sql.execution.streaming.runtime.StreamingCheckpointConstants.{DIR_NAME_COMMITS, DIR_NAME_OFFSETS, DIR_NAME_STATE}
 import org.apache.spark.sql.execution.streaming.sources.{ForeachBatchSink, WriteToMicroBatchDataSource, WriteToMicroBatchDataSourceV1}
-import org.apache.spark.sql.execution.streaming.state.{StateSchemaBroadcast, StateStoreErrors}
+import org.apache.spark.sql.execution.streaming.state.{OfflineStateRepartitionUtils, StateSchemaBroadcast, StateStoreErrors}
 import org.apache.spark.sql.internal.SQLConf
+import org.apache.spark.sql.internal.connector.PartitionOffsetWithIndex
 import org.apache.spark.sql.streaming.Trigger
 import org.apache.spark.util.{Clock, Utils}
 
@@ -74,6 +77,7 @@ class MicroBatchExecution(
       progressReporter,
       -1,
       sparkSession,
+      offsetLogFormatVersionOpt = None,
       previousContext = None)
 
   override def getLatestExecutionContext(): StreamExecutionContext = latestExecutionContext
@@ -97,6 +101,14 @@ class MicroBatchExecution(
 
   @volatile protected var sources: Seq[SparkDataStream] = Seq.empty
 
+  // Source ID mapping for OffsetMap support
+  // Using index as sourceId initially, can be extended to support user-provided names
+  // This is initialized in the same path as the sources Seq (defined above) and is used
+  // in the same way, when OffsetLog v2 is used.
+  @volatile protected var sourceIdMap: Map[String, SparkDataStream] = Map.empty
+
+  override protected def sourceToIdMap: Map[SparkDataStream, String] = sourceIdMap.map(_.swap)
+
   @volatile protected[sql] var triggerExecutor: TriggerExecutor = _
 
   protected def getTrigger(): TriggerExecutor = {
@@ -104,6 +116,7 @@ class MicroBatchExecution(
     trigger match {
       case t: ProcessingTimeTrigger => ProcessingTimeExecutor(t, triggerClock)
       case OneTimeTrigger => SingleBatchExecutor()
+      case _: RealTimeTrigger => ProcessingTimeExecutor(ProcessingTimeTrigger(0), triggerClock)
       case AvailableNowTrigger =>
         // When the flag is enabled, Spark will wrap sources which do not support
         // Trigger.AvailableNow with wrapper implementation, so that Trigger.AvailableNow can
@@ -172,7 +185,8 @@ class MicroBatchExecution(
 
     import org.apache.spark.sql.execution.datasources.v2.DataSourceV2Implicits._
     val _logicalPlan = analyzedPlan.transform {
-      case streamingRelation @ StreamingRelation(dataSourceV1, sourceName, output) =>
+      case streamingRelation @ StreamingRelation(
+          dataSourceV1, sourceName, output, sourceIdentifyingName) =>
         toExecutionRelationMap.getOrElseUpdate(streamingRelation, {
           // Materialize source to avoid creating it in every batch
           val metadataPath = s"$resolvedCheckpointRoot/sources/$nextSourceId"
@@ -181,11 +195,12 @@ class MicroBatchExecution(
           logInfo(log"Using Source [${MDC(LogKeys.STREAMING_SOURCE, source)}] " +
             log"from DataSourceV1 named '${MDC(LogKeys.STREAMING_DATA_SOURCE_NAME, sourceName)}' " +
             log"[${MDC(LogKeys.STREAMING_DATA_SOURCE_DESCRIPTION, dataSourceV1)}]")
-          StreamingExecutionRelation(source, output, dataSourceV1.catalogTable)(sparkSession)
+          StreamingExecutionRelation(
+            source, output, dataSourceV1.catalogTable, sourceIdentifyingName)(sparkSession)
         })
 
       case s @ StreamingRelationV2(src, srcName, table: SupportsRead, options, output,
-        catalog, identifier, v1) =>
+        catalog, identifier, v1, sourceIdentifyingName) =>
         val dsStr = if (src.nonEmpty) s"[${src.get}]" else ""
         val v2Disabled = disabledSources.contains(src.getOrElse(None).getClass.getCanonicalName)
         if (!v2Disabled && table.supports(TableCapability.MICRO_BATCH_READ)) {
@@ -200,7 +215,17 @@ class MicroBatchExecution(
             val scan = table.newScanBuilder(options).build()
             val stream = scan.toMicroBatchStream(metadataPath)
             val relation = StreamingDataSourceV2Relation(
-              table, output, catalog, identifier, options, metadataPath)
+                table,
+                output,
+                catalog,
+                identifier,
+                options,
+                metadataPath,
+                trigger match {
+                  case RealTimeTrigger(duration) => Some(duration)
+                  case _ => None
+                }
+              )
             StreamingDataSourceV2ScanRelation(relation, scan, output, stream)
           })
         } else if (v1.isEmpty) {
@@ -218,7 +243,7 @@ class MicroBatchExecution(
               log"${MDC(LogKeys.STREAMING_DATA_SOURCE_DESCRIPTION, dsStr)}")
             // We don't have a catalog table but may have a table identifier. Given this is about
             // v1 fallback path, we just give up and set the catalog table as None.
-            StreamingExecutionRelation(source, output, None)(sparkSession)
+            StreamingExecutionRelation(source, output, None, sourceIdentifyingName)(sparkSession)
           })
         }
     }
@@ -229,6 +254,24 @@ class MicroBatchExecution(
       case r: StreamingDataSourceV2ScanRelation => r.stream
     }
 
+    // Create source ID mapping for OffsetMap support
+    sourceIdMap = sources.zipWithIndex.map {
+      case (source, index) => index.toString -> source
+    }.toMap
+
+    // Inform the source if it is in real time mode
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      sources.foreach{
+        case s: SupportsRealTimeMode =>
+          s.prepareForRealTimeMode()
+        case s =>
+          throw new SparkIllegalArgumentException(
+            errorClass = "STREAMING_REAL_TIME_MODE.INPUT_STREAM_NOT_SUPPORTED",
+            messageParameters = Map("className" -> s.getClass.getName)
+          )
+      }
+    }
+
     // Initializing TriggerExecutor relies on `sources`, hence calling this after initializing
     // sources.
     triggerExecutor = getTrigger()
@@ -263,6 +306,12 @@ class MicroBatchExecution(
           case s => s -> ReadLimit.allAvailable()
         }.toMap
     }
+    if (trigger.isInstanceOf[RealTimeTrigger] && uniqueSources.size != sources.size) {
+      throw new SparkIllegalStateException(
+        errorClass = s"STREAMING_REAL_TIME_MODE.IDENTICAL_SOURCES_IN_UNION_NOT_SUPPORTED",
+        messageParameters = Map("sources" -> sources.mkString(", "))
+      )
+    }
 
     // TODO (SPARK-27484): we should add the writing node before the plan is analyzed.
     sink match {
@@ -279,6 +328,15 @@ class MicroBatchExecution(
           outputMode)
 
       case s: Sink =>
+        // SinkV1 is not compatible with Real-Time Mode due to API limitations.
+        // SinkV1 does not support writing outputs row by row.
+        if (trigger.isInstanceOf[RealTimeTrigger]) {
+          throw new SparkIllegalArgumentException(
+            errorClass = "STREAMING_REAL_TIME_MODE.SINK_NOT_SUPPORTED",
+            messageParameters = Map("className" -> s.getClass.getName)
+          )
+        }
+
         WriteToMicroBatchDataSourceV1(
           plan.catalogTable,
           sink = s,
@@ -333,20 +391,121 @@ class MicroBatchExecution(
 
   private def initializeExecution(
       sparkSessionForStream: SparkSession): MicroBatchExecutionContext = {
+    var latestStartedBatch = offsetLog.getLatest()
+    val latestCommittedBatch = commitLog.getLatest()
+
+    val lastCommittedBatchId = latestCommittedBatch match {
+      case Some((batchId, _)) => batchId
+      case _ => -1L
+    }
+
+    // Fail the query if the latest started batch is an uncommitted repartition batch
+    if (sparkSessionForStream.sessionState.conf.streamingCheckUnfinishedRepartitionOnRestart) {
+      checkUnfinishedRepartitionBatch(
+        latestStartedBatch.map(_._1), lastCommittedBatchId, offsetLog)
+    }
+
+    // For a query running in Real-time Mode that fails after
+    // writing to offset log but before writing to commit log, we delete the extra
+    // entries in offsetLog to sync up. Note that this also means async checkpoint rollback handling
+    // is not compatible with Real-time Mode at this stage.
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      val lastOffsetLogBatchId = latestStartedBatch.map(_._1).getOrElse(-1L)
+      if (lastOffsetLogBatchId > lastCommittedBatchId) {
+        logWarning(log"Deleting extra entries in offset log to sync up with commit log. " +
+          log"Last committed batch id = ${MDC(BATCH_ID, lastCommittedBatchId)}, " +
+          log"last offset log batch id = ${MDC(CURRENT_BATCH_ID, lastOffsetLogBatchId)}")
+        offsetLog.purgeAfter(lastCommittedBatchId)
+        latestStartedBatch = offsetLog.getLatest()
+      }
+    }
+
     AcceptsLatestSeenOffsetHandler.setLatestSeenOffsetOnSources(
-      offsetLog.getLatest().map(_._2), sources)
+      offsetLog.getLatest().map(_._2),
+      sources,
+      sourceIdMap
+    )
+
+    // Read the offset log format version from the last written offset log entry. If no entries
+    // are found, use the set/default value from the config.
+    val offsetLogFormatVersion = if (latestStartedBatch.isDefined) {
+      latestStartedBatch.get._2.version
+    } else {
+      // If no offset log entries are found, assert that the query does not have any committed
+      // batches to be extra safe.
+      assert(lastCommittedBatchId == -1L)
+      sparkSessionForStream.conf.get(SQLConf.STREAMING_OFFSET_LOG_FORMAT_VERSION)
+    }
+
+    // Set the offset log format version in the sparkSessionForStream conf
+    sparkSessionForStream.conf.set(
+      SQLConf.STREAMING_OFFSET_LOG_FORMAT_VERSION.key, offsetLogFormatVersion)
 
     val execCtx = new MicroBatchExecutionContext(id, runId, name, triggerClock, sources, sink,
-      progressReporter, -1, sparkSession, None)
+      progressReporter, -1, sparkSession,
+      offsetLogFormatVersionOpt = Some(offsetLogFormatVersion),
+      previousContext = None)
 
-    execCtx.offsetSeqMetadata =
-      OffsetSeqMetadata(batchWatermarkMs = 0, batchTimestampMs = 0, sparkSessionForStream.conf)
+    execCtx.offsetSeqMetadata = offsetLogFormatVersion match {
+      case OffsetSeqLog.VERSION_2 =>
+        OffsetSeqMetadataV2(batchWatermarkMs = 0, batchTimestampMs = 0, sparkSessionForStream.conf)
+      case OffsetSeqLog.VERSION_1 =>
+        OffsetSeqMetadata(batchWatermarkMs = 0, batchTimestampMs = 0, sparkSessionForStream.conf)
+      case v =>
+        throw QueryExecutionErrors.logVersionGreaterThanSupported(v, OffsetSeqLog.MAX_VERSION)
+    }
     setLatestExecutionContext(execCtx)
 
     populateStartOffsets(execCtx, sparkSessionForStream)
+
+    // SPARK-53941: This code path is executed for the first batch, regardless of whether it's a
+    // fresh new run or restart.
+    disableAQESupportInStatelessIfUnappropriated(sparkSessionForStream)
+
     logInfo(log"Stream started from ${MDC(LogKeys.STREAMING_OFFSETS_START, execCtx.startOffsets)}")
     execCtx
   }
+
+  protected def checkUnfinishedRepartitionBatch(
+      latestStartedBatchId: Option[Long],
+      lastCommittedBatchId: Long,
+      offsetLog: OffsetSeqLog): Unit = {
+    if (latestStartedBatchId.isDefined && lastCommittedBatchId >= 0 &&
+      latestStartedBatchId.get > lastCommittedBatchId &&
+      OfflineStateRepartitionUtils.isRepartitionBatch(latestStartedBatchId.get, offsetLog)) {
+      throw QueryExecutionErrors
+        .unfinishedRepartitionDetectedError(latestStartedBatchId.get, lastCommittedBatchId)
+    }
+  }
+
+  private def disableAQESupportInStatelessIfUnappropriated(
+      sparkSessionToRunBatches: SparkSession): Unit = {
+    def containsStatefulOperator(p: LogicalPlan): Boolean = {
+      p.exists {
+        case node: Aggregate if node.isStreaming => true
+        case node: Deduplicate if node.isStreaming => true
+        case node: DeduplicateWithinWatermark if node.isStreaming => true
+        case node: Distinct if node.isStreaming => true
+        case node: Join if node.left.isStreaming && node.right.isStreaming => true
+        case node: FlatMapGroupsWithState if node.isStreaming => true
+        case node: FlatMapGroupsInPandasWithState if node.isStreaming => true
+        case node: TransformWithState if node.isStreaming => true
+        case node: TransformWithStateInPySpark if node.isStreaming => true
+        case node: GlobalLimit if node.isStreaming => true
+        case _ => false
+      }
+    }
+
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      logWarning(log"Disabling AQE since AQE is not supported for Real-time Mode.")
+      sparkSessionToRunBatches.conf.set(SQLConf.ADAPTIVE_EXECUTION_ENABLED.key, "false")
+    } else if (containsStatefulOperator(analyzedPlan)) {
+      // SPARK-53941: We disable AQE for stateful workloads as of now.
+      logWarning(log"Disabling AQE since AQE is not supported in stateful workloads.")
+      sparkSessionToRunBatches.conf.set(SQLConf.ADAPTIVE_EXECUTION_ENABLED.key, "false")
+    }
+  }
+
   /**
    * Repeatedly attempts to run batches as data arrives.
    */
@@ -396,7 +555,12 @@ class MicroBatchExecution(
         // Record the trigger offset range for progress reporting *before* processing the batch
         execCtx.recordTriggerOffsets(
           from = execCtx.startOffsets,
-          to = execCtx.endOffsets,
+          to = if (trigger.isInstanceOf[RealTimeTrigger]) {
+            // We don't know endOffsets in real time mode here.
+            new StreamProgress(Map())
+          } else {
+            execCtx.endOffsets
+          },
           latest = execCtx.latestOffsets)
 
         // Remember whether the current batch has data or not. This will be required later
@@ -448,7 +612,7 @@ class MicroBatchExecution(
    * @param latestBatchId the batch id of the current micro batch
    * @return A option that contains the offset of the previously written batch
    */
-  def validateOffsetLogAndGetPrevOffset(latestBatchId: Long): Option[OffsetSeq] = {
+  def validateOffsetLogAndGetPrevOffset(latestBatchId: Long): Option[OffsetSeqBase] = {
     if (latestBatchId != 0) {
       Some(offsetLog.get(latestBatchId - 1).getOrElse {
         logError(log"The offset log for batch ${MDC(LogKeys.BATCH_ID, latestBatchId - 1)} " +
@@ -497,16 +661,16 @@ class MicroBatchExecution(
          * in the offset log */
         execCtx.batchId = latestBatchId
         execCtx.isCurrentBatchConstructed = true
-        execCtx.endOffsets = nextOffsets.toStreamProgress(sources)
+        execCtx.endOffsets = nextOffsets.toStreamProgress(sources, sourceIdMap)
 
         // validate the integrity of offset log and get the previous offset from the offset log
         val secondLatestOffsets = validateOffsetLogAndGetPrevOffset(latestBatchId)
         secondLatestOffsets.foreach { offset =>
-          execCtx.startOffsets = offset.toStreamProgress(sources)
+          execCtx.startOffsets = offset.toStreamProgress(sources, sourceIdMap)
         }
 
         // update offset metadata
-        nextOffsets.metadata.foreach { metadata =>
+        nextOffsets.metadataOpt.foreach { metadata =>
           OffsetSeqMetadata.setSessionConf(metadata, sparkSessionToRunBatches.sessionState.conf)
           execCtx.offsetSeqMetadata = OffsetSeqMetadata(
             metadata.batchWatermarkMs, metadata.batchTimestampMs, sparkSessionToRunBatches.conf)
@@ -603,12 +767,17 @@ class MicroBatchExecution(
    * Returns true if there is any new data available to be processed.
    */
   private def isNewDataAvailable(execCtx: MicroBatchExecutionContext): Boolean = {
-    execCtx.endOffsets.exists {
-      case (source, available) =>
-        execCtx.startOffsets
-          .get(source)
-          .map(committed => committed != available)
-          .getOrElse(true)
+    // For real-time mode, we always assume there is new data and run the batch.
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      true
+    } else {
+      execCtx.endOffsets.exists {
+        case (source, available) =>
+          execCtx.startOffsets
+            .get(source)
+            .map(committed => committed != available)
+            .getOrElse(true)
+      }
     }
   }
 
@@ -688,9 +857,22 @@ class MicroBatchExecution(
       .map(p => p._1 -> p._2.get).toMap
 
     // Update the query metadata
-    execCtx.offsetSeqMetadata = execCtx.offsetSeqMetadata.copy(
-      batchWatermarkMs = watermarkTracker.currentWatermark,
-      batchTimestampMs = triggerClock.getTimeMillis())
+    execCtx.offsetSeqMetadata = execCtx.offsetLogFormatVersionOpt.get match {
+      case OffsetSeqLog.VERSION_2 =>
+        OffsetSeqMetadataV2(
+          batchWatermarkMs = watermarkTracker.currentWatermark,
+          batchTimestampMs = triggerClock.getTimeMillis(),
+          conf = execCtx.offsetSeqMetadata.conf
+        )
+      case OffsetSeqLog.VERSION_1 =>
+        OffsetSeqMetadata(
+          batchWatermarkMs = watermarkTracker.currentWatermark,
+          batchTimestampMs = triggerClock.getTimeMillis(),
+          conf = execCtx.offsetSeqMetadata.conf
+        )
+      case v =>
+        throw QueryExecutionErrors.logVersionGreaterThanSupported(v, OffsetSeqLog.MAX_VERSION)
+    }
 
     // Check whether next batch should be constructed
     val lastExecutionRequiresAnotherBatch = noDataBatchesEnabled &&
@@ -737,8 +919,8 @@ class MicroBatchExecution(
     shouldConstructNextBatch
   }
 
-  protected def commitSources(offsetSeq: OffsetSeq): Unit = {
-    offsetSeq.toStreamProgress(sources).foreach {
+  protected def commitSources(offsetSeq: OffsetSeqBase): Unit = {
+    offsetSeq.toStreamProgress(sources, sourceIdMap).foreach {
       case (src: Source, off: Offset) => src.commit(off)
       case (stream: MicroBatchStream, off) =>
         stream.commit(stream.deserializeOffset(off.json))
@@ -770,6 +952,19 @@ class MicroBatchExecution(
           logDebug(s"Retrieving data from $source: $current -> $available")
           Some(source -> batch.logicalPlan)
 
+        case (stream: MicroBatchStream, _) if trigger.isInstanceOf[RealTimeTrigger] =>
+          if (!stream.isInstanceOf[SupportsRealTimeMode]) {
+            throw new SparkIllegalArgumentException(
+              errorClass = "STREAMING_REAL_TIME_MODE.INPUT_STREAM_NOT_SUPPORTED",
+              messageParameters = Map("className" -> stream.getClass.getName)
+            )
+          }
+          val current = execCtx.startOffsets.get(stream).map {
+            off => stream.deserializeOffset(off.json)
+          }
+          val startOffset = current.getOrElse(stream.initialOffset)
+          Some(stream -> OffsetHolder(startOffset, None))
+
         case (stream: MicroBatchStream, available)
           if execCtx.startOffsets.get(stream).map(_ != available).getOrElse(true) =>
           val current = execCtx.startOffsets.get(stream).map {
@@ -785,7 +980,7 @@ class MicroBatchExecution(
           // To be compatible with the v1 source, the `newData` is represented as a logical plan,
           // while the `newData` of v2 source is just the start and end offsets. Here we return a
           // fake logical plan to carry the offsets.
-          Some(stream -> OffsetHolder(startOffset, endOffset))
+          Some(stream -> OffsetHolder(startOffset, Some(endOffset)))
 
         case _ => None
       }
@@ -794,7 +989,7 @@ class MicroBatchExecution(
     // Replace sources in the logical plan with data that has arrived since the last batch.
     val newBatchesPlan = logicalPlan transform {
       // For v1 sources.
-      case StreamingExecutionRelation(source, output, catalogTable) =>
+      case StreamingExecutionRelation(source, output, catalogTable, _) =>
         mutableNewData.get(source).map { dataPlan =>
           val hasFileMetadata = output.exists {
             case FileSourceMetadataAttribute(_) => true
@@ -861,7 +1056,7 @@ class MicroBatchExecution(
       case r: StreamingDataSourceV2ScanRelation =>
         mutableNewData.get(r.stream).map {
           case OffsetHolder(start, end) =>
-            r.copy(startOffset = Some(start), endOffset = Some(end))
+            r.copy(startOffset = Some(start), endOffset = end)
         }.getOrElse {
           // Don't track the source node which is known to produce zero rows.
           LocalRelation(r.output, isStreaming = true)
@@ -923,6 +1118,14 @@ class MicroBatchExecution(
 
     markMicroBatchExecutionStart(execCtx)
 
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      RealTimeModeAllowlist.checkAllowedPhysicalOperator(
+        execCtx.executionPlan.executedPlan,
+        sparkSession.sessionState.conf.getConf(
+          SQLConf.STREAMING_REAL_TIME_MODE_ALLOWLIST_CHECK)
+      )
+    }
+
     if (execCtx.previousContext.isEmpty) {
       purgeStatefulMetadataAsync(execCtx.executionPlan.executedPlan)
     }
@@ -935,6 +1138,12 @@ class MicroBatchExecution(
       SQLExecution.withNewExecutionId(execCtx.executionPlan) {
         sink match {
           case s: Sink =>
+            if (trigger.isInstanceOf[RealTimeTrigger]) {
+              throw new SparkIllegalStateException(
+                errorClass = "STREAMING_REAL_TIME_MODE.SINK_NOT_SUPPORTED",
+                messageParameters = Map("className" -> s.getClass.getName)
+              )
+            }
             s.addBatch(execCtx.batchId, nextBatch)
             // DSv2 write node has a mechanism to invalidate DSv2 relation, but there is no
             // corresponding one for DSv1. Given we have an information of catalog table for sink,
@@ -966,13 +1175,23 @@ class MicroBatchExecution(
    * checkpointing to offset log and any microbatch startup tasks.
    */
   protected def markMicroBatchStart(execCtx: MicroBatchExecutionContext): Unit = {
-    if (!offsetLog.add(execCtx.batchId,
-      execCtx.endOffsets.toOffsetSeq(sources, execCtx.offsetSeqMetadata))) {
-      throw QueryExecutionErrors.concurrentStreamLogUpdate(execCtx.batchId)
-    }
 
-    logInfo(log"Committed offsets for batch ${MDC(LogKeys.BATCH_ID, execCtx.batchId)}. " +
-      log"Metadata ${MDC(LogKeys.OFFSET_SEQUENCE_METADATA, execCtx.offsetSeqMetadata.toString)}")
+    if (!trigger.isInstanceOf[RealTimeTrigger]) {
+      if (!offsetLog.add(
+          execCtx.batchId,
+          execCtx.endOffsets.toOffsets(sources, sourceIdMap, execCtx.offsetSeqMetadata)
+        )) {
+        throw QueryExecutionErrors.concurrentStreamLogUpdate(execCtx.batchId)
+      }
+
+      logInfo(
+        log"Committed offsets for batch ${MDC(LogKeys.BATCH_ID, execCtx.batchId)}. " +
+        log"Metadata ${MDC(LogKeys.OFFSET_SEQUENCE_METADATA, execCtx.offsetSeqMetadata.toString)}"
+      )
+    } else {
+      logInfo(log"Delay offset logging for batch ${MDC(BATCH_ID, execCtx.batchId)} " +
+        log"in real time mode.")
+    }
   }
 
   /**
@@ -1078,6 +1297,72 @@ class MicroBatchExecution(
       sparkSessionForStream.sessionState.conf)) {
       updateStateStoreCkptId(execCtx, latestExecPlan)
     }
+
+    var needSignalProgressLock = false
+    // In real-time mode, we delay the offset logging until the end of the batch.
+    // We first gather the offsets processed up to from all RealTimeStreamScanExec,
+    // i.e. tasks that execute a source partition.  We merge the offsets and
+    // write them to the offset log
+    if (trigger.isInstanceOf[RealTimeTrigger]) {
+      val execs = StreamingQueryPlanTraverseHelper
+        .collectFromUnfoldedPlan(lastExecution.executedPlan) {
+        case e: RealTimeStreamScanExec => e
+      }
+
+      val endOffsetMap = MutableMap[SparkDataStream, OffsetV2]()
+      execs.foreach { e =>
+        val lowLatencyExec = e.asInstanceOf[RealTimeStreamScanExec]
+        val accus: Seq[PartitionOffsetWithIndex] =
+          lowLatencyExec.endOffsetsAccumulator.value.asScala.toSeq
+        val sortedPartitionOffsets = accus.sortBy(_.index).map(_.partitionOffset).toArray
+        val source = e.stream
+        val endOffset = source
+          .asInstanceOf[SupportsRealTimeMode]
+          .mergeOffsets(sortedPartitionOffsets)
+        endOffsetMap += (source -> endOffset)
+      }
+
+      assert(endOffsetMap.size == execs.size, "Identical sources exist in the physical nodes" +
+        " which is not supported.")
+
+      execCtx.endOffsets ++= endOffsetMap
+      execCtx.recordEndOffsets(execCtx.endOffsets)
+      execCtx.recordTriggerOffsets(
+        from = execCtx.startOffsets,
+        to = execCtx.endOffsets,
+        latest = execCtx.latestOffsets
+      )
+      execCtx.reportTimeTaken("walCommit") {
+        if (!offsetLog.add(
+          execCtx.batchId,
+          execCtx.endOffsets.toOffsets(sources, sourceIdMap, execCtx.offsetSeqMetadata)
+          )) {
+          throw QueryExecutionErrors.concurrentStreamLogUpdate(execCtx.batchId)
+        }
+      }
+      logInfo(
+        log"Committed offsets for batch ${MDC(LogKeys.BATCH_ID, execCtx.batchId)}. Metadata " +
+        log"${MDC(LogKeys.OFFSET_SEQUENCE_METADATA, execCtx.offsetSeqMetadata)}"
+      )
+      var shouldUpdate = true
+      sources.foreach { s =>
+        execCtx.startOffsets.get(s).foreach { prevOffsets =>
+          if (!prevOffsets.equals(endOffsetMap(s))) {
+            shouldUpdate = false
+          }
+        }
+      }
+      if (shouldUpdate) {
+        // To trigger processAllAvailable() return.
+        noNewData = true
+
+        // We could signal ProcessAllAvailable to finish here, however
+        // signaling after commit log will make it less likely that the caller of
+        // ProcessAllAvailable() sees offset log written but not commit log.
+        needSignalProgressLock = true
+      }
+    }
+
     execCtx.reportTimeTaken("commitOffsets") {
       val stateStoreCkptId = if (StatefulOperatorStateInfo.enableStateStoreCheckpointIds(
         sparkSessionForStream.sessionState.conf)) {
@@ -1091,6 +1376,15 @@ class MicroBatchExecution(
       }
     }
     committedOffsets ++= execCtx.endOffsets
+
+    // RealTime Mode deals with ProcessAllAvailable() differently. It sets noNewData above
+    // when a batch ends, so we need to signal here. Non-Real-Time mode sets the same flag
+    // in query planning phase.
+    if (needSignalProgressLock) {
+      withProgressLocked {
+        awaitProgressLockCondition.signalAll()
+      }
+    }
   }
 
   protected def cleanUpLastExecutedMicroBatch(execCtx: MicroBatchExecutionContext): Unit = {
@@ -1122,6 +1416,6 @@ object MicroBatchExecution {
   val BATCH_ID_KEY = "streaming.sql.batchId"
 }
 
-case class OffsetHolder(start: OffsetV2, end: OffsetV2) extends LeafNode {
+case class OffsetHolder(start: OffsetV2, end: Option[OffsetV2]) extends LeafNode {
   override def output: Seq[Attribute] = Nil
 }
