@@ -365,6 +365,32 @@ class JsonFunctionsSuite extends QueryTest with SharedSparkSession {
       Row(Row(1, "haa")) :: Nil)
   }
 
+  test("from_json with NULL in options map values") {
+    val df = Seq("""{"str": "World"}""").toDS()
+    val schema = "str STRING"
+
+    checkAnswer(
+      df.selectExpr(
+        s"from_json(value, '$schema', map('key', 'value', 'mode', NULL))"
+      ),
+      Row(Row("World")) :: Nil
+    )
+  }
+
+  test("from_json with NULL in options map key") {
+    val df = Seq("""{"str": "World"}""").toDS()
+    val schema = "str STRING"
+
+    checkError(
+      exception = intercept[SparkRuntimeException] {
+        df.selectExpr(
+          s"from_json(value, '$schema', map('mode', 'PERMISSIVE', NULL, 'value'))"
+        ).show()
+      },
+      condition = "NULL_MAP_KEY"
+    )
+  }
+
   test("to_json - struct") {
     val df = Seq(Tuple1(Tuple1(1))).toDF("a")
 
@@ -1479,4 +1505,86 @@ class JsonFunctionsSuite extends QueryTest with SharedSparkSession {
       checkAnswer(df4, Row("1", "2", "3"))
     }
   }
+
+  test("from_json/to_json with TIME type - all precisions") {
+    import java.time.LocalTime
+    // Test data: (precision, LocalTime, expected JSON string)
+    val testData = Seq(
+      (0, LocalTime.of(14, 30, 45), "14:30:45"),
+      (1, LocalTime.of(14, 30, 45, 100000000), "14:30:45.1"),
+      (2, LocalTime.of(14, 30, 45, 120000000), "14:30:45.12"),
+      (3, LocalTime.of(14, 30, 45, 123000000), "14:30:45.123"),
+      (4, LocalTime.of(14, 30, 45, 123400000), "14:30:45.1234"),
+      (5, LocalTime.of(14, 30, 45, 123450000), "14:30:45.12345"),
+      (6, LocalTime.of(14, 30, 45, 123456000), "14:30:45.123456")
+    )
+
+    testData.foreach { case (precision, time, timeStr) =>
+      val schema = new StructType().add("time", TimeType(precision))
+
+      // Test from_json
+      val parseResult = Seq(s"""{"time": "$timeStr"}""").toDS()
+        .select(from_json($"value", schema)).collect().head.getAs[Row](0).getAs[LocalTime](0)
+      assert(parseResult == time, s"from_json failed for precision $precision")
+
+      // Test to_json
+      val jsonResult = Seq(time).toDF("time")
+        .select(to_json(struct($"time"))).collect().head.getString(0)
+      assert(jsonResult == s"""{"time":"$timeStr"}""", s"to_json failed for precision $precision")
+
+      // Test roundtrip
+      val roundtrip = Seq(time).toDF("time")
+        .select(to_json(struct($"time")).as("json"))
+        .select(from_json($"json", schema).as("struct"))
+        .select($"struct.time").collect().head.getAs[LocalTime](0)
+      assert(roundtrip == time, s"Roundtrip failed for precision $precision")
+    }
+
+    // Test custom format
+    val schema = new StructType().add("time", TimeType(6))
+    val customTime = LocalTime.of(14, 30, 45, 123456000)
+    val customFormat = Map("timeFormat" -> "HH-mm-ss.SSSSSS")
+
+    checkAnswer(
+      Seq("""{"time": "14-30-45.123456"}""").toDS()
+        .select(from_json($"value", schema, customFormat).as("data"))
+        .select($"data.time"),
+      Row(customTime) :: Nil)
+
+    checkAnswer(
+      Seq(customTime).toDF("time").select(to_json(struct($"time"), customFormat)),
+      Row("""{"time":"14-30-45.123456"}""") :: Nil)
+  }
+
+  test("TIME type with arrays and nulls") {
+    import java.time.LocalTime
+
+    // Test array support
+    val times = Seq(LocalTime.of(9, 0, 0), LocalTime.of(17, 45, 30))
+    val schema = new StructType().add("times", ArrayType(TimeType()))
+    val jsonStr = """{"times":["09:00:00","17:45:30"]}"""
+
+    // to_json
+    checkAnswer(
+      Seq(times).toDF("times").select(to_json(struct($"times"))),
+      Row(jsonStr) :: Nil)
+
+    // from_json
+    val parsed = Seq(jsonStr).toDS()
+      .select(from_json($"value", schema).as("data"))
+      .select($"data.times").collect().head.getSeq[LocalTime](0)
+    assert(parsed == times)
+
+    // Test null handling
+    checkAnswer(
+      Seq("""{"time": null}""").toDS()
+        .select(from_json($"value", new StructType().add("time", TimeType())).as("data"))
+        .select($"data.time"),
+      Row(null) :: Nil)
+
+    // Verify schema inference treats TIME strings as STRING (like TIMESTAMP)
+    val inferredSchema = spark.sql("SELECT schema_of_json('{\"time\": \"14:30:45\"}')")
+      .collect().head.getString(0)
+    assert(inferredSchema.contains("STRING"))
+  }
 }
