@@ -19,7 +19,8 @@ package org.apache.spark.sql
 
 import scala.collection.mutable.ArrayBuffer
 
-import org.apache.spark.sql.catalyst.expressions.SubqueryExpression
+import org.apache.spark.SparkRuntimeException
+import org.apache.spark.sql.catalyst.expressions.{EqualTo, NamedExpression, OuterReference, SubqueryExpression}
 import org.apache.spark.sql.catalyst.plans.{LeftAnti, LeftSemi}
 import org.apache.spark.sql.catalyst.plans.logical.{Aggregate, Filter, Join, LogicalPlan, Project, Sort, Union}
 import org.apache.spark.sql.execution._
@@ -527,43 +528,30 @@ class SubquerySuite extends QueryTest
   test("SPARK-18504 extra GROUP BY column in correlated scalar subquery is not permitted") {
     withTempView("v") {
       Seq((1, 1), (1, 2)).toDF("c1", "c2").createOrReplaceTempView("v")
-
-      val exception = intercept[AnalysisException] {
-        sql("select (select sum(-1) from v t2 where t1.c2 = t2.c1 group by t2.c2) sum from v t1")
+      val exception = intercept[SparkRuntimeException] {
+        sql("select (select sum(-1) from v t2 where t1.c2 = t2.c1 group by t2.c2) sum from v t1").
+          collect()
       }
       checkError(
         exception,
-        condition = "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
-          "NON_CORRELATED_COLUMNS_IN_GROUP_BY",
-        parameters = Map("value" -> "c2"),
-        sqlState = None,
-        context = ExpectedContext(
-          fragment = "(select sum(-1) from v t2 where t1.c2 = t2.c1 group by t2.c2)",
-          start = 7, stop = 67)) }
+        condition = "SCALAR_SUBQUERY_TOO_MANY_ROWS"
+      )
+    }
   }
 
   test("non-aggregated correlated scalar subquery") {
-    val exception1 = intercept[AnalysisException] {
-      sql("select a, (select b from l l2 where l2.a = l1.a) sum_b from l l1")
+    val exception1 = intercept[SparkRuntimeException] {
+      sql("select a, (select b from l l2 where l2.a = l1.a) sum_b from l l1").collect()
     }
     checkError(
       exception1,
-      condition = "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
-        "MUST_AGGREGATE_CORRELATED_SCALAR_SUBQUERY",
-      parameters = Map.empty,
-      context = ExpectedContext(
-        fragment = "(select b from l l2 where l2.a = l1.a)", start = 10, stop = 47))
-    val exception2 = intercept[AnalysisException] {
-      sql("select a, (select b from l l2 where l2.a = l1.a group by 1) sum_b from l l1")
-    }
-    checkErrorMatchPVals(
-      exception2,
-      condition = "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
-        "MUST_AGGREGATE_CORRELATED_SCALAR_SUBQUERY",
-      parameters = Map.empty[String, String],
-      sqlState = None,
-      context = ExpectedContext(
-        fragment = "(select b from l l2 where l2.a = l1.a group by 1)", start = 10, stop = 58))
+      condition = "SCALAR_SUBQUERY_TOO_MANY_ROWS"
+    )
+    checkAnswer(
+      sql("select a, (select b from l l2 where l2.a = l1.a group by 1) sum_b from l l1"),
+      Row(1, 2.0) :: Row(1, 2.0) :: Row(2, 1.0) :: Row(2, 1.0) :: Row(3, 3.0) ::
+        Row(null, null) :: Row(null, null) :: Row(6, null) :: Nil
+    )
   }
 
   test("non-equal correlated scalar subquery") {
@@ -937,12 +925,12 @@ class SubquerySuite extends QueryTest
 
       withSQLConf(SQLConf.DECORRELATE_INNER_QUERY_ENABLED.key -> "false") {
         val error = intercept[AnalysisException] { sql(query) }
-        assert(error.getErrorClass == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
+        assert(error.getCondition == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
           "ACCESSING_OUTER_QUERY_COLUMN_IS_NOT_ALLOWED")
       }
       withSQLConf(SQLConf.DECORRELATE_SET_OPS_ENABLED.key -> "false") {
         val error = intercept[AnalysisException] { sql(query) }
-        assert(error.getErrorClass == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
+        assert(error.getCondition == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
           "ACCESSING_OUTER_QUERY_COLUMN_IS_NOT_ALLOWED")
       }
 
@@ -1016,12 +1004,12 @@ class SubquerySuite extends QueryTest
 
           withSQLConf(SQLConf.DECORRELATE_INNER_QUERY_ENABLED.key -> "false") {
             val error = intercept[AnalysisException] { sql(query) }
-            assert(error.getErrorClass == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
+            assert(error.getCondition == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
               "ACCESSING_OUTER_QUERY_COLUMN_IS_NOT_ALLOWED")
           }
           withSQLConf(SQLConf.DECORRELATE_SET_OPS_ENABLED.key -> "false") {
             val error = intercept[AnalysisException] { sql(query) }
-            assert(error.getErrorClass == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
+            assert(error.getCondition == "UNSUPPORTED_SUBQUERY_EXPRESSION_CATEGORY." +
               "ACCESSING_OUTER_QUERY_COLUMN_IS_NOT_ALLOWED")
           }
         }
@@ -1536,7 +1524,7 @@ class SubquerySuite extends QueryTest
       // need to execute the query before we can examine fs.inputRDDs()
       assert(stripAQEPlan(df.queryExecution.executedPlan) match {
         case WholeStageCodegenExec(ColumnarToRowExec(InputAdapter(
-            fs @ FileSourceScanExec(_, _, _, partitionFilters, _, _, _, _, _)))) =>
+            fs @ FileSourceScanExec(_, _, _, _, partitionFilters, _, _, _, _, _)))) =>
           partitionFilters.exists(ExecSubqueryExpression.hasSubquery) &&
             fs.inputRDDs().forall(
               _.asInstanceOf[FileScanRDD].filePartitions.forall(
@@ -2154,6 +2142,24 @@ class SubquerySuite extends QueryTest
     }
   }
 
+  test("SPARK-49819: Do not collapse projects with exist subqueries") {
+    withTempView("v") {
+      Seq((0, 1), (1, 2)).toDF("c1", "c2").createOrReplaceTempView("v")
+      checkAnswer(
+        sql("""
+              |SELECT m, CASE WHEN EXISTS (SELECT SUM(c2) FROM v WHERE c1 = m) THEN 1 ELSE 0 END
+              |FROM (SELECT MIN(c2) AS m FROM v)
+              |""".stripMargin),
+        Row(1, 1) :: Nil)
+      checkAnswer(
+        sql("""
+              |SELECT c, CASE WHEN EXISTS (SELECT SUM(c2) FROM v WHERE c1 = c) THEN 1 ELSE 0 END
+              |FROM (SELECT c1 AS c FROM v GROUP BY c1)
+              |""".stripMargin),
+        Row(0, 1) :: Row(1, 1) :: Nil)
+    }
+  }
+
   test("SPARK-37199: deterministic in QueryPlan considers subquery") {
     val deterministicQueryPlan = sql("select (select 1 as b) as b")
       .queryExecution.executedPlan
@@ -2228,161 +2234,6 @@ class SubquerySuite extends QueryTest
     }
   }
 
-  test("Merge non-correlated scalar subqueries") {
-    Seq(false, true).foreach { enableAQE =>
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> enableAQE.toString) {
-        val df = sql(
-          """
-            |SELECT
-            |  (SELECT avg(key) FROM testData),
-            |  (SELECT sum(key) FROM testData),
-            |  (SELECT count(distinct key) FROM testData)
-          """.stripMargin)
-
-        checkAnswer(df, Row(50.5, 5050, 100) :: Nil)
-
-        val plan = df.queryExecution.executedPlan
-        val subqueryIds = collectWithSubqueries(plan) { case s: SubqueryExec => s.id }
-        val reusedSubqueryIds = collectWithSubqueries(plan) {
-          case rs: ReusedSubqueryExec => rs.child.id
-        }
-
-        assert(subqueryIds.size == 1, "Missing or unexpected SubqueryExec in the plan")
-        assert(reusedSubqueryIds.size == 2,
-          "Missing or unexpected reused ReusedSubqueryExec in the plan")
-      }
-    }
-  }
-
-  test("Merge non-correlated scalar subqueries in a subquery") {
-    Seq(false, true).foreach { enableAQE =>
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> enableAQE.toString) {
-        val df = sql(
-          """
-            |SELECT (
-            |  SELECT
-            |    SUM(
-            |      (SELECT avg(key) FROM testData) +
-            |      (SELECT sum(key) FROM testData) +
-            |      (SELECT count(distinct key) FROM testData))
-            |   FROM testData
-            |)
-          """.stripMargin)
-
-        checkAnswer(df, Row(520050.0) :: Nil)
-
-        val plan = df.queryExecution.executedPlan
-        val subqueryIds = collectWithSubqueries(plan) { case s: SubqueryExec => s.id }
-        val reusedSubqueryIds = collectWithSubqueries(plan) {
-          case rs: ReusedSubqueryExec => rs.child.id
-        }
-
-        assert(subqueryIds.size == 2, "Missing or unexpected SubqueryExec in the plan")
-        assert(reusedSubqueryIds.size == 5,
-          "Missing or unexpected reused ReusedSubqueryExec in the plan")
-      }
-    }
-  }
-
-  test("Merge non-correlated scalar subqueries from different levels") {
-    Seq(false, true).foreach { enableAQE =>
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> enableAQE.toString) {
-        val df = sql(
-          """
-            |SELECT
-            |  (SELECT avg(key) FROM testData),
-            |  (
-            |    SELECT
-            |      SUM(
-            |        (SELECT sum(key) FROM testData)
-            |      )
-            |    FROM testData
-            |  )
-          """.stripMargin)
-
-        checkAnswer(df, Row(50.5, 505000) :: Nil)
-
-        val plan = df.queryExecution.executedPlan
-        val subqueryIds = collectWithSubqueries(plan) { case s: SubqueryExec => s.id }
-        val reusedSubqueryIds = collectWithSubqueries(plan) {
-          case rs: ReusedSubqueryExec => rs.child.id
-        }
-
-        assert(subqueryIds.size == 2, "Missing or unexpected SubqueryExec in the plan")
-        assert(reusedSubqueryIds.size == 2,
-          "Missing or unexpected reused ReusedSubqueryExec in the plan")
-      }
-    }
-  }
-
-  test("Merge non-correlated scalar subqueries from different parent plans") {
-    Seq(false, true).foreach { enableAQE =>
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> enableAQE.toString) {
-        val df = sql(
-          """
-            |SELECT
-            |  (
-            |    SELECT
-            |      SUM(
-            |        (SELECT avg(key) FROM testData)
-            |      )
-            |    FROM testData
-            |  ),
-            |  (
-            |    SELECT
-            |      SUM(
-            |        (SELECT sum(key) FROM testData)
-            |      )
-            |    FROM testData
-            |  )
-          """.stripMargin)
-
-        checkAnswer(df, Row(5050.0, 505000) :: Nil)
-
-        val plan = df.queryExecution.executedPlan
-        val subqueryIds = collectWithSubqueries(plan) { case s: SubqueryExec => s.id }
-        val reusedSubqueryIds = collectWithSubqueries(plan) {
-          case rs: ReusedSubqueryExec => rs.child.id
-        }
-
-        assert(subqueryIds.size == 2, "Missing or unexpected SubqueryExec in the plan")
-        assert(reusedSubqueryIds.size == 4,
-          "Missing or unexpected reused ReusedSubqueryExec in the plan")
-      }
-    }
-  }
-
-  test("Merge non-correlated scalar subqueries with conflicting names") {
-    Seq(false, true).foreach { enableAQE =>
-      withSQLConf(
-        SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> enableAQE.toString) {
-        val df = sql(
-          """
-            |SELECT
-            |  (SELECT avg(key) AS key FROM testData),
-            |  (SELECT sum(key) AS key FROM testData),
-            |  (SELECT count(distinct key) AS key FROM testData)
-          """.stripMargin)
-
-        checkAnswer(df, Row(50.5, 5050, 100) :: Nil)
-
-        val plan = df.queryExecution.executedPlan
-        val subqueryIds = collectWithSubqueries(plan) { case s: SubqueryExec => s.id }
-        val reusedSubqueryIds = collectWithSubqueries(plan) {
-          case rs: ReusedSubqueryExec => rs.child.id
-        }
-
-        assert(subqueryIds.size == 1, "Missing or unexpected SubqueryExec in the plan")
-        assert(reusedSubqueryIds.size == 2,
-          "Missing or unexpected reused ReusedSubqueryExec in the plan")
-      }
-    }
-  }
-
   test("SPARK-39355: Single column uses quoted to construct UnresolvedAttribute") {
     checkAnswer(
       sql("""
@@ -2483,39 +2334,6 @@ class SubquerySuite extends QueryTest
     }
   }
 
-  test("SPARK-40618: Regression test for merging subquery bug with nested subqueries") {
-    // This test contains a subquery expression with another subquery expression nested inside.
-    // It acts as a regression test to ensure that the MergeScalarSubqueries rule does not attempt
-    // to merge them together.
-    withTable("t1", "t2") {
-      sql("create table t1(col int) using csv")
-      checkAnswer(sql("select(select sum((select sum(col) from t1)) from t1)"), Row(null))
-
-      checkAnswer(sql(
-        """
-          |select
-          |  (select sum(
-          |    (select sum(
-          |        (select sum(col) from t1))
-          |     from t1))
-          |  from t1)
-          |""".stripMargin),
-        Row(null))
-
-      sql("create table t2(col int) using csv")
-      checkAnswer(sql(
-        """
-          |select
-          |  (select sum(
-          |    (select sum(
-          |        (select sum(col) from t1))
-          |     from t2))
-          |  from t1)
-          |""".stripMargin),
-        Row(null))
-    }
-  }
-
   test("SPARK-40615: Check unsupported data type when decorrelating subqueries") {
     withTempView("v1", "v2") {
       sql(
@@ -2556,9 +2374,9 @@ class SubquerySuite extends QueryTest
             "UNSUPPORTED_CORRELATED_REFERENCE_DATA_TYPE",
           parameters = Map("expr" -> "v1.x", "dataType" -> "map"),
           context = ExpectedContext(
-            fragment = "select upper(x['a'] + rand()) as a",
-            start = 39,
-            stop = 72)
+            fragment = "(\n  select concat(a, a) from\n  (select upper(x['a'] + rand()) as a)\n)",
+            start = 7,
+            stop = 75)
         )
       }
     }
@@ -2610,31 +2428,6 @@ class SubquerySuite extends QueryTest
     }
   }
 
-  test("SPARK-42346: Rewrite distinct aggregates after merging subqueries") {
-    withTempView("t1") {
-      Seq((1, 2), (3, 4)).toDF("c1", "c2").createOrReplaceTempView("t1")
-
-      checkAnswer(sql(
-        """
-          |SELECT
-          |  (SELECT count(distinct c1) FROM t1),
-          |  (SELECT count(distinct c2) FROM t1)
-          |""".stripMargin),
-        Row(2, 2))
-
-      // In this case we don't merge the subqueries as `RewriteDistinctAggregates` kicks off for the
-      // 2 subqueries first but `MergeScalarSubqueries` is not prepared for the `Expand` nodes that
-      // are inserted by the rewrite.
-      checkAnswer(sql(
-        """
-          |SELECT
-          |  (SELECT count(distinct c1) + sum(distinct c2) FROM t1),
-          |  (SELECT count(distinct c2) + sum(distinct c1) FROM t1)
-          |""".stripMargin),
-        Row(8, 6))
-    }
-  }
-
   test("SPARK-42745: Improved AliasAwareOutputExpression works with DSv2") {
     withSQLConf(
       SQLConf.USE_V1_SOURCE_LIST.key -> "") {
@@ -2716,7 +2509,7 @@ class SubquerySuite extends QueryTest
           |SELECT * FROM v1 WHERE kind = (SELECT kind FROM v1 WHERE kind = 'foo')
           |""".stripMargin)
       val df = sql("SELECT * FROM v1 JOIN v2 ON v1.id = v2.id")
-      val filter = df.queryExecution.optimizedPlan.collect {
+      val filter = df.queryExecution.optimizedPlan.collectFirst {
         case f: Filter => f
       }
       assert(filter.isEmpty,
@@ -2794,4 +2587,84 @@ class SubquerySuite extends QueryTest
       checkAnswer(df3, Row(7))
     }
   }
+
+  test("SPARK-50091: Handle aggregates in left-hand operand of IN-subquery") {
+    withView("v1", "v2") {
+      Seq((1, 2, 2), (1, 5, 3), (2, 0, 4), (3, 7, 7), (3, 8, 8))
+        .toDF("c1", "c2", "c3")
+        .createOrReplaceTempView("v1")
+      Seq((1, 2, 2), (1, 3, 3), (2, 2, 4), (3, 7, 7), (3, 1, 1))
+        .toDF("col1", "col2", "col3")
+        .createOrReplaceTempView("v2")
+
+      val df1 = sql("SELECT col1, SUM(col2) IN (SELECT c3 FROM v1) FROM v2 GROUP BY col1")
+      checkAnswer(df1,
+        Row(1, false) :: Row(2, true) :: Row(3, true) :: Nil)
+
+      val df2 = sql("""SELECT
+                      |  col1,
+                      |  SUM(col2) IN (SELECT c3 FROM v1) and SUM(col3) IN (SELECT c2 FROM v1) AS x
+                      |FROM v2 GROUP BY col1
+                      |ORDER BY col1""".stripMargin)
+      checkAnswer(df2,
+        Row(1, false) :: Row(2, false) :: Row(3, true) :: Nil)
+
+      val df3 = sql("""SELECT col1, (SUM(col2), SUM(col3)) IN (SELECT c3, c2 FROM v1) AS x
+                      |FROM v2
+                      |GROUP BY col1
+                      |ORDER BY col1""".stripMargin)
+      checkAnswer(df3,
+        Row(1, false) :: Row(2, false) :: Row(3, true) :: Nil)
+    }
+  }
+
+  test("SPARK-51738: IN subquery with struct type") {
+    checkAnswer(
+      sql("SELECT foo IN (SELECT struct(1 a)) FROM (SELECT struct(1 b) foo)"),
+      Row(true)
+    )
+
+    checkAnswer(
+      sql("""
+            |SELECT foo IN (SELECT struct(c, d) FROM r)
+            |FROM (SELECT struct(a, b) foo FROM l)
+            |""".stripMargin),
+      Row(false) :: Row(false) :: Row(false) :: Row(false) :: Row(false)
+        :: Row(true) :: Row(true) :: Row(true) :: Nil
+    )
+  }
+
+
+  test("SPARK-52896: Outer reference ExprId should match exposed attribute") {
+    val plan =
+      sql(
+        """
+          | SELECT col1
+          | FROM VALUES(1,2)
+          | GROUP BY col1
+          | HAVING MAX(col2) == (SELECT 1 WHERE MAX(col2) = 1)
+          |
+      """.stripMargin).queryExecution.analyzed
+
+    // Expected plan:
+    // Project
+    // +- Filter (scalar-subquery)
+    // :  +- Project
+    // :     +- Filter
+    // :        +- OneRowRelation
+    // +- Aggregate
+    //   +- LocalRelation
+
+    val havingNode = plan.asInstanceOf[Project].child.asInstanceOf[Filter]
+    val subquery =
+      havingNode.condition.asInstanceOf[EqualTo].right.asInstanceOf[SubqueryExpression]
+    val subqueryFilter = subquery.plan.asInstanceOf[Project].child.asInstanceOf[Filter]
+
+    val exposedAttribute = subquery.getOuterAttrs.head.asInstanceOf[NamedExpression]
+    val outerReferenceAttribute = subqueryFilter.condition.asInstanceOf[EqualTo].collectFirst {
+      case outerReference: OuterReference => outerReference.e
+    }.get
+
+    assert(exposedAttribute.exprId == outerReferenceAttribute.exprId)
+  }
 }
