@@ -36,7 +36,7 @@ groups = {
     "bitwise_funcs", "conversion_funcs", "csv_funcs",
     "xml_funcs", "lambda_funcs", "collection_funcs",
     "url_funcs", "hash_funcs", "struct_funcs",
-    "table_funcs", "variant_funcs"
+    "table_funcs", "variant_funcs", "protobuf_funcs", "sketch_funcs"
 }
 
 
@@ -175,7 +175,7 @@ def _make_pretty_examples(jspark, infos):
     pretty_output = ""
     for info in infos:
         if (info.examples.startswith("\n    Examples:") and info.name.lower() not in
-                ("from_avro", "to_avro", "from_protobuf", "to_protobuf")):
+                ("from_avro", "to_avro", "from_protobuf", "to_protobuf", "measure")):
             output = []
             output.append("-- %s" % info.name)
             query_examples = filter(lambda x: x.startswith("      > "), info.examples.split("\n"))
@@ -240,6 +240,8 @@ def generate_functions_examples_html(jvm, jspark, html_output_dir):
     </pre></div>
 
     """
+    print("Enabling TIME data type")
+    jspark.sql("SET spark.sql.timeType.enabled = true")
     print("Running SQL examples to generate formatted output.")
     for key, infos in _list_grouped_function_infos(jvm):
         examples = _make_pretty_examples(jspark, infos)
