@@ -20,6 +20,7 @@ import scala.collection.mutable
 
 import org.apache.spark.{SparkException, SparkRuntimeException}
 import org.apache.spark.sql.{AnalysisException, SparkSession}
+import org.apache.spark.sql.catalyst.QueryPlanningTracker
 import org.apache.spark.sql.catalyst.analysis.UnresolvedRelation
 import org.apache.spark.sql.catalyst.plans.logical.{CreateFlowCommand, CreateMaterializedViewAsSelect, CreateStreamingTable, CreateStreamingTableAsSelect, CreateView, InsertIntoStatement, LogicalPlan}
 import org.apache.spark.sql.catalyst.util.StringUtils
@@ -111,21 +112,19 @@ class SqlGraphRegistrationContext(
   )
 
   def processSqlFile(sqlText: String, sqlFilePath: String, spark: SparkSession): Unit = {
-    // Create a registration context for this SQL registration request
-    val sqlGraphElementRegistrationContext = new SqlGraphRegistrationContext(
-      graphRegistrationContext
-    )
-
     splitSqlFileIntoQueries(
       spark = spark,
       sqlFileText = sqlText,
       sqlFilePath = sqlFilePath
     ).foreach { case SqlQueryPlanWithOrigin(logicalPlan, queryOrigin) =>
-      sqlGraphElementRegistrationContext.processSqlQuery(logicalPlan, queryOrigin)
+      processSqlQuery(logicalPlan, queryOrigin, spark)
     }
   }
 
-  private def processSqlQuery(queryPlan: LogicalPlan, queryOrigin: QueryOrigin): Unit = {
+  private def processSqlQuery(
+      queryPlan: LogicalPlan,
+      queryOrigin: QueryOrigin,
+      spark: SparkSession): Unit = {
     queryPlan match {
       case setCommand: SetCommand =>
         // SET [ key | 'key' ] [ value | 'value' ]
@@ -144,7 +143,7 @@ class SqlGraphRegistrationContext(
         // objects such as tables are resolved from said catalog, until overwritten, within this
         // SQL processor scope. Note that the schema is cleared when the catalog is set, and must
         // be explicitly set again in order to implicitly qualify identifiers.
-        SetCatalogCommandHandler.handle(setCatalogCommand)
+        SetCatalogCommandHandler.handle(setCatalogCommand, queryOrigin, spark)
       case createPersistedViewCommand: CreateView =>
         // CREATE VIEW [ persisted_view_name ] [ options ] AS [ query ]
         CreatePersistedViewCommandHandler.handle(createPersistedViewCommand, queryOrigin)
@@ -192,8 +191,9 @@ class SqlGraphRegistrationContext(
           specifiedSchema =
             Option.when(cst.columns.nonEmpty)(StructType(cst.columns.map(_.toV1Column))),
           partitionCols = Option(PartitionHelper.applyPartitioning(cst.partitioning, queryOrigin)),
+          clusterCols = None,
           properties = cst.tableSpec.properties,
-          baseOrigin = queryOrigin.copy(
+          origin = queryOrigin.copy(
             objectName = Option(stIdentifier.unquotedString),
             objectType = Option(QueryOriginType.Table.toString)
           ),
@@ -223,8 +223,9 @@ class SqlGraphRegistrationContext(
           specifiedSchema =
             Option.when(cst.columns.nonEmpty)(StructType(cst.columns.map(_.toV1Column))),
           partitionCols = Option(PartitionHelper.applyPartitioning(cst.partitioning, queryOrigin)),
+          clusterCols = None,
           properties = cst.tableSpec.properties,
-          baseOrigin = queryOrigin.copy(
+          origin = queryOrigin.copy(
             objectName = Option(stIdentifier.unquotedString),
             objectType = Option(QueryOriginType.Table.toString)
           ),
@@ -273,8 +274,9 @@ class SqlGraphRegistrationContext(
           specifiedSchema =
             Option.when(cmv.columns.nonEmpty)(StructType(cmv.columns.map(_.toV1Column))),
           partitionCols = Option(PartitionHelper.applyPartitioning(cmv.partitioning, queryOrigin)),
+          clusterCols = None,
           properties = cmv.tableSpec.properties,
-          baseOrigin = queryOrigin.copy(
+          origin = queryOrigin.copy(
             objectName = Option(mvIdentifier.unquotedString),
             objectType = Option(QueryOriginType.Table.toString)
           ),
@@ -413,7 +415,7 @@ class SqlGraphRegistrationContext(
         )
         .identifier
 
-      val (flowTargetDatasetIdentifier, flowQueryLogicalPlan, isOnce) = cf.flowOperation match {
+      val (flowTargetDatasetIdentifier, flowQueryLogicalPlan) = cf.flowOperation match {
         case i: InsertIntoStatement =>
           validateInsertIntoFlow(i, queryOrigin)
           val flowTargetDatasetName = i.table match {
@@ -432,7 +434,7 @@ class SqlGraphRegistrationContext(
               currentDatabase = context.getCurrentDatabaseOpt
             )
             .identifier
-          (qualifiedFlowTargetDatasetName, i.query, false)
+          (qualifiedFlowTargetDatasetName, i.query)
         case _ =>
           throw SqlGraphElementRegistrationException(
             msg = "Unable flow type. Only INSERT INTO flows are supported.",
@@ -454,7 +456,7 @@ class SqlGraphRegistrationContext(
           destinationIdentifier = qualifiedDestinationIdentifier,
           func = FlowAnalysis.createFlowFunctionFromLogicalPlan(flowQueryLogicalPlan),
           sqlConf = context.getSqlConf,
-          once = isOnce,
+          once = false,
           queryContext = QueryContext(
             currentCatalog = context.getCurrentCatalogOpt,
             currentDatabase = context.getCurrentDatabaseOpt
@@ -503,11 +505,10 @@ class SqlGraphRegistrationContext(
 
   private object SetCommandHandler {
     def handle(setCommand: SetCommand): Unit = {
-      val sqlConfKvPair = setCommand.kv.getOrElse(
+      val (sqlConfKey, valueOpt) = setCommand.kv.getOrElse(
         throw new RuntimeException("Invalid SET command without key-value pair")
       )
-      val sqlConfKey = sqlConfKvPair._1
-      val sqlConfValue = sqlConfKvPair._2.getOrElse(
+      val sqlConfValue = valueOpt.getOrElse(
         throw new RuntimeException("Invalid SET command without value")
       )
       context.setSqlConf(sqlConfKey, sqlConfValue)
@@ -532,8 +533,24 @@ class SqlGraphRegistrationContext(
   }
 
   private object SetCatalogCommandHandler {
-    def handle(setCatalogCommand: SetCatalogCommand): Unit = {
-      context.setCurrentCatalog(setCatalogCommand.catalogName)
+    def handle(
+        setCatalogCommand: SetCatalogCommand,
+        queryOrigin: QueryOrigin,
+        spark: SparkSession): Unit = {
+      try {
+        // Analyze unresolved references before handling the command.
+        val analyzed = spark.sessionState.analyzer.executeAndCheck(
+          setCatalogCommand,
+          new QueryPlanningTracker
+        ).asInstanceOf[SetCatalogCommand]
+        context.setCurrentCatalog(analyzed.getCatalogName())
+      } catch {
+        case e: AnalysisException =>
+          throw SqlGraphElementRegistrationException(
+            msg = s"Failed to resolve catalog expression: ${e.getMessage}",
+            queryOrigin = queryOrigin
+          )
+      }
       context.clearCurrentDatabase()
     }
   }
